/*Asic_mid.c: function of Asic set;*/
/* sane - Scanner Access Now Easy.

   Copyright (C) 2005 Mustek.
   Originally maintained by Mustek
   Author:Roy 2005.5.24

   This file is part of the SANE package.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston,
   MA 02111-1307, USA.

   As a special exception, the authors of SANE give permission for
   additional uses of the libraries contained in this release of SANE.

   The exception is that, if you link a SANE library with other files
   to produce an executable, this does not by itself cause the
   resulting executable to be covered by the GNU General Public
   License.  Your use of that executable is in no way restricted on
   account of linking the SANE library code into it.

   This exception does not, however, invalidate any other reasons why
   the executable file might be covered by the GNU General Public
   License.

   If you submit changes to SANE to the maintainers to be included in
   a subsequent release, you agree by submitting the changes that
   those changes may be distributed with this exception intact.

   If you write modifications of your own for SANE, it is your choice
   whether to permit this exception to apply to your modifications.
   If you do not wish that, delete this exception notice.

   This file implements a SANE backend for Mustek 1200UB and similar 
   USB flatbed scanners.  */

#include "mustek_usb2_asic_low.c"

/* forward declarations */
extern STATUS OpenScanChip(PAsic chip);

extern STATUS CloseScanChip(PAsic chip);

extern STATUS SafeInitialChip(PAsic chip);

extern STATUS DRAM_Test(PAsic chip);

extern STATUS SetPowerSave(PAsic chip);

extern STATUS SetLineTimeAndExposure(PAsic chip);

extern STATUS CCDTiming(PAsic chip);

extern STATUS IsCarriageHome(PAsic chip,BOOL* LampHome ,BOOL* TAHome);

extern STATUS InitTiming(PAsic chip);

extern STATUS GetChipStatus(PAsic chip,BYTE Selector, BYTE *ChipStatus);

extern STATUS SetAFEGainOffset(PAsic chip);

extern STATUS SetLEDTime(PAsic chip);

extern STATUS SetScanMode(PAsic chip,BYTE bScanBits);

extern STATUS SetPackAddress(PAsic chip,WORD wXResolution, 
			     WORD wWidth,WORD wX,double XRatioAdderDouble,
			     double XRatioTypeDouble, BYTE byClear_Pulse_Width,
			     WORD *PValidPixelNumber);

extern STATUS SetExtraSetting(PAsic chip,WORD wXResolution,WORD  wCCD_PixelNumber,BOOL isCaribrate);


STATUS InitTiming(PAsic chip)
{
	STATUS status = STATUS_GOOD;
	DBG (DBG_ASIC, "InitTiming:Enter\n");

	chip->Timing.AFE_ADCCLK_Timing		=1010580480;
	chip->Timing.AFE_ADCVS_Timing		=12582912;
	chip->Timing.AFE_ADCRS_Timing		=3072;
	chip->Timing.AFE_ChannelA_LatchPos	=3080;
	chip->Timing.AFE_ChannelB_LatchPos	=3602;
	chip->Timing.AFE_ChannelC_LatchPos	=5634;
	chip->Timing.AFE_ChannelD_LatchPos	=1546;
	chip->Timing.AFE_Secondary_FF_LatchPos=12;
	
	/* Sensor */
	chip->Timing.CCD_DummyCycleTiming	=0;
	chip->Timing.PHTG_PluseWidth		=12;
	chip->Timing.PHTG_WaitWidth			=1;
	chip->Timing.PHTG_TimingAdj			=1;
	chip->Timing.PHTG_TimingSetup		=0;
	chip->Timing.ChannelR_StartPixel	=100;
	chip->Timing.ChannelR_EndPixel		=200;
	chip->Timing.ChannelG_StartPixel	=100;
	chip->Timing.ChannelG_EndPixel		=200;
	chip->Timing.ChannelB_StartPixel	=100;
	chip->Timing.ChannelB_EndPixel		=200;
	
	/*1200dpi Timing*/
	chip->Timing.CCD_PH2_Timing_1200		=1048320;
	chip->Timing.CCD_PHRS_Timing_1200		=983040;
	chip->Timing.CCD_PHCP_Timing_1200		=61440;
	chip->Timing.CCD_PH1_Timing_1200		=4293918720;
	chip->Timing.DE_CCD_SETUP_REGISTER_1200 =32;
	chip->Timing.wCCDPixelNumber_1200		=11250;
	
    /*600dpi Timing*/
	chip->Timing.CCD_PH2_Timing_600			=1048320;
	chip->Timing.CCD_PHRS_Timing_600	    =983040;
	chip->Timing.CCD_PHCP_Timing_600	    =61440;
	chip->Timing.CCD_PH1_Timing_600			=4293918720;
	chip->Timing.DE_CCD_SETUP_REGISTER_600  =0;
	chip->Timing.wCCDPixelNumber_600		=7500;

	DBG (DBG_ASIC, "InitTiming:Exit\n");
	return status;
}

STATUS OpenScanChip(PAsic chip)
{
	STATUS status = STATUS_GOOD;
	DBG (DBG_ASIC, "OpenScanChip:Enter\n");

	BYTE x[4];
	
	x[0] = 0x64;
	x[1] = 0x64;
	x[2] = 0x64;
	x[3] = 0x64;
	status = WriteIOControl(chip, 0x90, 0, 4, x);
	if (status != STATUS_GOOD)
		return status;
	
	x[0] = 0x65;
	x[1] = 0x65;
	x[2] = 0x65;
	x[3] = 0x65;
	status = WriteIOControl(chip, 0x90, 0, 4, x);
	if (status != STATUS_GOOD)
		return status;
	
	x[0] = 0x44;
	x[1] = 0x44;
	x[2] = 0x44;
	x[3] = 0x44;
	status = WriteIOControl(chip, 0x90, 0, 4, x);
	if (status != STATUS_GOOD)
		return status;
	
	x[0] = 0x45;
	x[1] = 0x45;
	x[2] = 0x45;
	x[3] = 0x45;
	status = WriteIOControl(chip, 0x90, 0, 4, x);
	
	DBG (DBG_ASIC, "OpenScanChip: Exit\n");
	return status;
}


STATUS CloseScanChip(PAsic chip)
{
	STATUS status = STATUS_GOOD;
	DBG (DBG_ASIC, "CloseScanChip:Enter\n");
	
	BYTE x[4];
	
	x[0] = 0x64;
	x[1] = 0x64;
	x[2] = 0x64;
	x[3] = 0x64;
	status = WriteIOControl(chip, 0x90, 0, 4, x);
	if (status != STATUS_GOOD)
		return status;
	
	x[0] = 0x65;
	x[1] = 0x65;
	x[2] = 0x65;
	x[3] = 0x65;
	status = WriteIOControl(chip, 0x90, 0, 4, x);
	if (status != STATUS_GOOD)
		return status;
	
	x[0] = 0x16;
	x[1] = 0x16;
	x[2] = 0x16;
	x[3] = 0x16;
	status = WriteIOControl(chip, 0x90, 0, 4, x);
	if (status != STATUS_GOOD)
		return status;
	
	x[0] = 0x17;
	x[1] = 0x17;
	x[2] = 0x17;
	x[3] = 0x17;
	status = WriteIOControl(chip, 0x90, 0, 4, x);
	
	DBG (DBG_ASIC, "CloseScanChip: Exit\n");
	return status;
}


STATUS SafeInitialChip(PAsic chip)
{
	STATUS status = STATUS_GOOD;
	DBG (DBG_ASIC, "SafeInitialChip:Enter\n");
	BYTE temp;
		
	Mustek_SendData(chip,ES01_F3_ActionOption, 0);
	Mustek_SendData(chip,ES01_86_DisableAllClockWhenIdle, CLOSE_ALL_CLOCK_DISABLE);
	Mustek_SendData(chip,ES01_F4_ActiveTriger,ACTION_TRIGER_DISABLE);
	
	status=Asic_WaitUnitReady(chip);
	
	DBG (DBG_ASIC, "isFirstOpenChip=%d\n",chip->isFirstOpenChip);
  	if(chip->isFirstOpenChip)
	{
	DBG (DBG_ASIC, "isFirstOpenChip=%d\n",chip->isFirstOpenChip);
		status = DRAM_Test(chip);
		if (status != STATUS_GOOD)
		{
			DBG (DBG_ASIC, "DRAM_Test: Error\n");
			return status;
		}
		chip->isFirstOpenChip=FALSE;
	}
	
	DBG (DBG_ASIC, "SafeInitialChip: exit\n");
	return status;
}


STATUS DRAM_Test(PAsic chip)
{
	STATUS status = STATUS_GOOD;
	DBG (DBG_ASIC, "DRAM_Test:Enter\n");

	unsigned char * temps;
	unsigned int	i;
	
	temps = (unsigned char *) malloc (64);
	
	for (i = 0; i < 64; i++) 
	{
		*(temps + i) = i;
	}
	
	/*set start address*/
	status = Mustek_SendData(chip, ES01_A0_HostStartAddr0_7, 0x00);
	if (status != STATUS_GOOD)
	{
		free(temps);
		return status;
	}
	
	status = Mustek_SendData(chip, ES01_A1_HostStartAddr8_15, 0x00);
	if (status != STATUS_GOOD)
	{
		free(temps);
		return status;
	}
	
	status = Mustek_SendData(chip, ES01_A2_HostStartAddr16_21, 0x00|ACCESS_DRAM);
	if (status != STATUS_GOOD)
	{
		free(temps);
		return status;
	}

	Mustek_SendData(chip,ES01_79_AFEMCLK_SDRAMCLK_DELAY_CONTROL,SDRAMCLK_DELAY_12_ns);
	status = Mustek_SendData(chip, ES01_A3_HostEndAddr0_7, 0xff);
	if (status != STATUS_GOOD)
	{
		free(temps);
		return status;
	}
	
	status = Mustek_SendData(chip, ES01_A4_HostEndAddr8_15, 0xff);
	if (status != STATUS_GOOD)
	{
		free(temps);
		return status;
	}
	
	status = Mustek_SendData(chip, ES01_A5_HostEndAddr16_21, 0xff);
	if (status != STATUS_GOOD)
	{
		free(temps);
		return status;
	}

	status = Mustek_DMAWrite(chip, 64, (LPBYTE)(temps));
	if (status != STATUS_GOOD)
	{
		DBG (DBG_ASIC, "Mustek_DMAWrite error\n");
		free(temps);
		return status;
	}
	
	status = Mustek_SendData(chip, ES01_A0_HostStartAddr0_7, 0x00);
	if (status != STATUS_GOOD)
	{
		free(temps);
		return status;
	}
	
	status = Mustek_SendData(chip, ES01_A1_HostStartAddr8_15, 0x00);
	if (status != STATUS_GOOD)
	{
		free(temps);
		return status;
	}
	
	status = Mustek_SendData(chip, ES01_A2_HostStartAddr16_21, 0x00| ACCESS_DRAM);
	if (status != STATUS_GOOD)
	{
		free(temps);
		return status;
	}
	
	/*set end address*/
	status = Mustek_SendData(chip, ES01_A3_HostEndAddr0_7, 0xff);
	if (status != STATUS_GOOD)
	{
		free(temps);
		return status;
	}
	
	status = Mustek_SendData(chip, ES01_A4_HostEndAddr8_15, 0xff);
	if (status != STATUS_GOOD)
	{
		free(temps);
		return status;
	}
	
	status = Mustek_SendData(chip, ES01_A5_HostEndAddr16_21, 0xff);
	if (status != STATUS_GOOD)
	{
		free(temps);
		return status;
	}
	
	memset(temps, 0, 64);

	status = Mustek_DMARead(chip, 64, (LPBYTE)(temps));
	if (status != STATUS_GOOD)
	{
		free(temps);
		return status;
	}

	for (i = 0; i <60; i+=10)
	{
     DBG (DBG_ASIC, "%d,%d,%d,%d,%d,%d,%d,%d,%d,%d\n",
		 *(temps +i),*(temps +i+1),*(temps +i+2),*(temps +i+3),*(temps +i+4),
		*(temps +i+5),*(temps +i+6),*(temps +i+7),*(temps +i+8),*(temps +i+9));
	}
  
	for (i = 0; i < 64; i++)
	{
		if (*(temps + i) != i)
		{
			fprintf(stderr, "DRAM Test error...(No.=%d)\n",i+1);
			return STATUS_IO_ERROR;
		}
	}
	
	free (temps);
	
	DBG (DBG_ASIC, "DRAM_Text: Exit\n");
	return status;
}


STATUS SetPowerSave(PAsic chip)
{
	STATUS status = STATUS_GOOD;
	DBG (DBG_ASIC, "SetPowerSave:Enter\n");

	if (chip->firmwarestate < FS_OPENED)
		OpenScanChip(chip);
	
	if (chip->firmwarestate > FS_OPENED)
		Asic_ScanStop(chip);
	
	Mustek_SendData (chip,ES01_94_PowerSaveControl, 0x10);
	
	chip->firmwarestate = FS_OPENED;
	DBG (DBG_ASIC, "SetPowerSave:Exit\n");
	return status;
}

STATUS SetLineTimeAndExposure(PAsic chip)
{
	STATUS status = STATUS_GOOD;
	DBG (DBG_ASIC, "SetLineTimeAndExposure:Enter\n");

	if (chip->firmwarestate < FS_OPENED)
		OpenScanChip(chip);
	
	Mustek_SendData(chip,ES01_C4_MultiTGTimesRed, 0);
	Mustek_SendData(chip,ES01_C5_MultiTGTimesGreen, 0);
	Mustek_SendData(chip,ES01_C6_MultiTGTimesBlue, 0);
	
	Mustek_SendData(chip,ES01_C7_MultiTGDummyPixelNumberLSB, 0);
	Mustek_SendData(chip,ES01_C8_MultiTGDummyPixelNumberMSB, 0);
	
	Mustek_SendData(chip,ES01_C9_CCDDummyPixelNumberLSB, 0);
	Mustek_SendData(chip,ES01_CA_CCDDummyPixelNumberMSB, 0);
	
	Mustek_SendData(chip,ES01_CB_CCDDummyCycleNumber, 0);
	

	chip->firmwarestate=FS_OPENED;
	
	DBG (DBG_ASIC, "SetLineTimeAndExposure:Exit\n");
	return status;
}





STATUS CCDTiming(PAsic chip)
{
	STATUS status = STATUS_GOOD;
	DBG (DBG_ASIC, "CCDTiming:Enter\n");
	DBG (DBG_ASIC, "Dpi=%d\n",chip->Scan.Dpi);

	if (chip->firmwarestate<FS_OPENED)
		OpenScanChip(chip);
	
	Mustek_SendData (chip,ES01_82_AFE_ADCCLK_TIMING_ADJ_BYTE0, (BYTE) (chip->Timing.AFE_ADCCLK_Timing) );
	Mustek_SendData (chip,ES01_83_AFE_ADCCLK_TIMING_ADJ_BYTE1, (BYTE) (chip->Timing.AFE_ADCCLK_Timing>>8) );
	Mustek_SendData (chip,ES01_84_AFE_ADCCLK_TIMING_ADJ_BYTE2, (BYTE) (chip->Timing.AFE_ADCCLK_Timing>>16) );
	Mustek_SendData (chip,ES01_85_AFE_ADCCLK_TIMING_ADJ_BYTE3, (BYTE) (chip->Timing.AFE_ADCCLK_Timing>>24) );

	Mustek_SendData (chip,ES01_1F0_AFERS_TIMING_ADJ_B0, (BYTE) (chip->Timing.AFE_ADCRS_Timing) );
	Mustek_SendData (chip,ES01_1F1_AFERS_TIMING_ADJ_B1, (BYTE) (chip->Timing.AFE_ADCRS_Timing>>8) );
	Mustek_SendData (chip,ES01_1F2_AFERS_TIMING_ADJ_B2, (BYTE) (chip->Timing.AFE_ADCRS_Timing>>16) );
	Mustek_SendData (chip,ES01_1F3_AFERS_TIMING_ADJ_B3, (BYTE) (chip->Timing.AFE_ADCRS_Timing>>24) );
	
	Mustek_SendData (chip,ES01_1EC_AFEVS_TIMING_ADJ_B0, (BYTE) (chip->Timing.AFE_ADCVS_Timing) );
	Mustek_SendData (chip,ES01_1ED_AFEVS_TIMING_ADJ_B1, (BYTE) (chip->Timing.AFE_ADCVS_Timing>>8) );
	Mustek_SendData (chip,ES01_1EE_AFEVS_TIMING_ADJ_B2, (BYTE) (chip->Timing.AFE_ADCVS_Timing>>16) );
	Mustek_SendData (chip,ES01_1EF_AFEVS_TIMING_ADJ_B3, (BYTE) (chip->Timing.AFE_ADCVS_Timing>>24) );
	
	Mustek_SendData (chip,ES01_160_CHANNEL_A_LATCH_POSITION_HB, HIBYTE (chip->Timing.AFE_ChannelA_LatchPos) );
	Mustek_SendData (chip,ES01_161_CHANNEL_A_LATCH_POSITION_LB, LOBYTE (chip->Timing.AFE_ChannelA_LatchPos) );
	
	Mustek_SendData (chip,ES01_162_CHANNEL_B_LATCH_POSITION_HB, HIBYTE (chip->Timing.AFE_ChannelB_LatchPos) );
	Mustek_SendData (chip,ES01_163_CHANNEL_B_LATCH_POSITION_LB, LOBYTE (chip->Timing.AFE_ChannelB_LatchPos) );
	
	Mustek_SendData (chip,ES01_164_CHANNEL_C_LATCH_POSITION_HB, HIBYTE (chip->Timing.AFE_ChannelC_LatchPos) );
	Mustek_SendData (chip,ES01_165_CHANNEL_C_LATCH_POSITION_LB, LOBYTE (chip->Timing.AFE_ChannelC_LatchPos) );
	
	Mustek_SendData (chip,ES01_166_CHANNEL_D_LATCH_POSITION_HB, HIBYTE (chip->Timing.AFE_ChannelD_LatchPos) );
	Mustek_SendData (chip,ES01_167_CHANNEL_D_LATCH_POSITION_LB, LOBYTE (chip->Timing.AFE_ChannelD_LatchPos) );
	
	Mustek_SendData (chip,ES01_168_SECONDARY_FF_LATCH_POSITION, chip->Timing.AFE_Secondary_FF_LatchPos );
	
	Mustek_SendData (chip,ES01_1D0_DUMMY_CYCLE_TIMING_B0, (BYTE) (chip->Timing.CCD_DummyCycleTiming) );
	Mustek_SendData (chip,ES01_1D1_DUMMY_CYCLE_TIMING_B1, (BYTE) (chip->Timing.CCD_DummyCycleTiming>>8) );
	Mustek_SendData (chip,ES01_1D2_DUMMY_CYCLE_TIMING_B2, (BYTE) (chip->Timing.CCD_DummyCycleTiming>>16) );
	Mustek_SendData (chip,ES01_1D3_DUMMY_CYCLE_TIMING_B3, (BYTE) (chip->Timing.CCD_DummyCycleTiming>>24) );
	
	DWORD dwPH1, dwPH2, dwPHRS, dwPHCP;
	if (chip->Scan.Dpi >= 1200)
	{
		dwPH1 = chip->Timing.CCD_PH1_Timing_1200;
		dwPH2 = chip->Timing.CCD_PH2_Timing_1200;
		dwPHRS = chip->Timing.CCD_PHRS_Timing_1200;
		dwPHCP = chip->Timing.CCD_PHCP_Timing_1200;
	}
	else
	{
		dwPH1 = chip->Timing.CCD_PH1_Timing_600;
		dwPH2 = chip->Timing.CCD_PH2_Timing_600;
		dwPHRS = chip->Timing.CCD_PHRS_Timing_600;
		dwPHCP = chip->Timing.CCD_PHCP_Timing_600;
	}
	
	Mustek_SendData (chip,ES01_1D4_PH1_TIMING_ADJ_B0, (BYTE) (dwPH1) );
	Mustek_SendData (chip,ES01_1D5_PH1_TIMING_ADJ_B1, (BYTE) (dwPH1>>8) );
	Mustek_SendData (chip,ES01_1D6_PH1_TIMING_ADJ_B2, (BYTE) (dwPH1>>16) );
	Mustek_SendData (chip,ES01_1D7_PH1_TIMING_ADJ_B3, (BYTE) (dwPH1>>24) );
	
	/* set ccd ph1 ph2 rs cp */
	Mustek_SendData (chip,ES01_D0_PH1_0, 0 );	
	Mustek_SendData(chip,ES01_D1_PH2_0,	4);
	Mustek_SendData(chip,ES01_D4_PHRS_0,0);
	Mustek_SendData(chip,ES01_D5_PHCP_0,0);
	
	Mustek_SendData (chip,ES01_1D8_PH2_TIMING_ADJ_B0, (BYTE) (dwPH2));   
	Mustek_SendData (chip,ES01_1D9_PH2_TIMING_ADJ_B1, (BYTE) (dwPH2>>8));  
	Mustek_SendData (chip,ES01_1DA_PH2_TIMING_ADJ_B2, (BYTE) (dwPH2>>16));  
	Mustek_SendData (chip,ES01_1DB_PH2_TIMING_ADJ_B3, (BYTE) (dwPH2>>24));  
	
	Mustek_SendData (chip,ES01_1E4_PHRS_TIMING_ADJ_B0, (BYTE) (dwPHRS));   
	Mustek_SendData (chip,ES01_1E5_PHRS_TIMING_ADJ_B1, (BYTE) (dwPHRS>>8)); 
	Mustek_SendData (chip,ES01_1E6_PHRS_TIMING_ADJ_B2, (BYTE) (dwPHRS>>16)); 
	Mustek_SendData (chip,ES01_1E7_PHRS_TIMING_ADJ_B3, (BYTE) (dwPHRS>>24)); 
	
	Mustek_SendData (chip,ES01_1E8_PHCP_TIMING_ADJ_B0, (BYTE) (dwPHCP));   
	Mustek_SendData (chip,ES01_1E9_PHCP_TIMING_ADJ_B1, (BYTE) (dwPHCP>>8));   
	Mustek_SendData (chip,ES01_1EA_PHCP_TIMING_ADJ_B2, (BYTE) (dwPHCP>>16));  
	Mustek_SendData (chip,ES01_1EB_PHCP_TIMING_ADJ_B3, (BYTE) (dwPHCP>>24));  
	
	chip->firmwarestate=FS_OPENED;
	DBG (DBG_ASIC, "CCDTiming:Exit\n");
	return status;
}

STATUS IsCarriageHome(PAsic chip,BOOL* LampHome ,BOOL* TAHome)
{
	STATUS status = STATUS_GOOD;
	DBG (DBG_ASIC, "IsCarriageHome:Enter\n");
	BYTE temp;
	
	status = GetChipStatus(chip,0, &temp);
	if (status != STATUS_GOOD)
	{
		DBG (DBG_ASIC, "IsCarriageHome:Error!\n");
		return status;
	}
	
	if((temp & SENSOR0_DETECTED)==SENSOR0_DETECTED)
		*LampHome = TRUE;
	else 
	{
		*LampHome = FALSE;
	}

	*TAHome=TRUE;

	DBG (DBG_ASIC, "LampHome=%d\n",*LampHome);

	DBG (DBG_ASIC, "IsCarriageHome:Exit\n");
	return status;
}


STATUS GetChipStatus(PAsic chip,BYTE Selector, BYTE *ChipStatus)
{
	STATUS status = STATUS_GOOD;
	DBG (DBG_ASIC, "GetChipStatus:Enter\n");
	
	status =Mustek_SendData(chip,ES01_8B_Status, Selector);
	if(status != STATUS_GOOD)
		return status;
	
	status =Mustek_WriteAddressLineForRegister(chip,ES01_8B_Status);
	if(status != STATUS_GOOD)
		return status;

	*ChipStatus = ES01_8B_Status;
	status =Mustek_ReceiveData(chip,ChipStatus);
	if(status != STATUS_GOOD)
		return status;
	
	DBG (DBG_ASIC, "GetChipStatus:Exit\n");
	return status;
}

STATUS SetAFEGainOffset(PAsic chip)
{
	STATUS status = STATUS_GOOD;
	DBG (DBG_ASIC, "SetAFEGainOffset:Enter\n");
	
	if (chip->AD.DirectionR)
	{/* Negative */
		Mustek_SendData(chip,ES01_60_AFE_AUTO_GAIN_OFFSET_RED_LB, (chip->AD.GainR<<1) | 0x01);
		Mustek_SendData(chip,ES01_61_AFE_AUTO_GAIN_OFFSET_RED_HB, chip->AD.OffsetR);
	}
	else
	{/* Postive */
		Mustek_SendData(chip,ES01_60_AFE_AUTO_GAIN_OFFSET_RED_LB, (chip->AD.GainR<<1));
		Mustek_SendData(chip,ES01_61_AFE_AUTO_GAIN_OFFSET_RED_HB, chip->AD.OffsetR);
	}

	if (chip->AD.DirectionG)
	{
		Mustek_SendData(chip,ES01_62_AFE_AUTO_GAIN_OFFSET_GREEN_LB, (chip->AD.GainG<<1) | 0x01);
		Mustek_SendData(chip,ES01_63_AFE_AUTO_GAIN_OFFSET_GREEN_HB, chip->AD.OffsetG);
	}
	else
	{
		Mustek_SendData(chip,ES01_62_AFE_AUTO_GAIN_OFFSET_GREEN_LB, (chip->AD.GainG<<1));

		Mustek_SendData(chip,ES01_63_AFE_AUTO_GAIN_OFFSET_GREEN_HB, chip->AD.OffsetG);
	}

	if (chip->AD.DirectionB)
	{
		Mustek_SendData(chip,ES01_64_AFE_AUTO_GAIN_OFFSET_BLUE_LB,	(chip->AD.GainB<<1) | 0x01);
		Mustek_SendData(chip,ES01_65_AFE_AUTO_GAIN_OFFSET_BLUE_HB,	chip->AD.OffsetB);
	}
	else
	{
		Mustek_SendData(chip,ES01_64_AFE_AUTO_GAIN_OFFSET_BLUE_LB,	(chip->AD.GainB<<1));
		Mustek_SendData(chip,ES01_65_AFE_AUTO_GAIN_OFFSET_BLUE_HB, chip->AD.OffsetB);
	}


	Mustek_SendData(chip,ES01_2A0_AFE_GAIN_OFFSET_CONTROL, 0x01);

	int i = 0;
	for (i=0 ; i<4 ; i++){

		if (chip->AD.DirectionR == 0)
		{
			Mustek_SendData(chip,ES01_2A1_AFE_AUTO_CONFIG_GAIN, (BYTE) (chip->AD.GainR<<1) );
			Mustek_SendData(chip,ES01_2A2_AFE_AUTO_CONFIG_OFFSET, (BYTE) (chip->AD.OffsetR) );
		}
		else
		{
			Mustek_SendData(chip,ES01_2A1_AFE_AUTO_CONFIG_GAIN, (BYTE) (chip->AD.GainR<<1) | 0x01 );
			Mustek_SendData(chip,ES01_2A2_AFE_AUTO_CONFIG_OFFSET, (BYTE) (chip->AD.OffsetR) );
		}
	}

	for (i=0 ; i<4 ; i++){
		if (chip->AD.DirectionG  == 0)
		{
			Mustek_SendData(chip,ES01_2A1_AFE_AUTO_CONFIG_GAIN, (BYTE) (chip->AD.GainG<<1) );
			Mustek_SendData(chip,ES01_2A2_AFE_AUTO_CONFIG_OFFSET, (BYTE) (chip->AD.OffsetG) );
		}
		else
		{
			Mustek_SendData(chip,ES01_2A1_AFE_AUTO_CONFIG_GAIN, (BYTE) (chip->AD.GainG<<1) | 0x01 );
			Mustek_SendData(chip,ES01_2A2_AFE_AUTO_CONFIG_OFFSET, (BYTE) (chip->AD.OffsetG) );
		}
	}

	for (i=0 ; i<4 ; i++){
		if (chip->AD.DirectionB == 0)
		{
			Mustek_SendData(chip,ES01_2A1_AFE_AUTO_CONFIG_GAIN, (BYTE) (chip->AD.GainB<<1) );
			Mustek_SendData(chip,ES01_2A2_AFE_AUTO_CONFIG_OFFSET, (BYTE) (chip->AD.OffsetB) );
		}
		else
		{
			Mustek_SendData(chip,ES01_2A1_AFE_AUTO_CONFIG_GAIN, (BYTE) (chip->AD.GainB<<1) | 0x01 );
			Mustek_SendData(chip,ES01_2A2_AFE_AUTO_CONFIG_OFFSET, (BYTE) (chip->AD.OffsetB) );
		}
	}

	for (i=0 ; i<36 ; i++)
	{
		Mustek_SendData(chip,ES01_2A1_AFE_AUTO_CONFIG_GAIN, 0);
		Mustek_SendData(chip,ES01_2A2_AFE_AUTO_CONFIG_OFFSET, 0);
	}

	Mustek_SendData(chip,ES01_2A0_AFE_GAIN_OFFSET_CONTROL, 0x00);

	/* Set to AFE */
	Mustek_SendData(chip,ES01_04_ADAFEPGACH1,	chip->AD.GainR  );
	Mustek_SendData(chip,ES01_06_ADAFEPGACH2,	chip->AD.GainG  );
	Mustek_SendData(chip,ES01_08_ADAFEPGACH3,	chip->AD.GainB  );

	if(chip->AD.DirectionR)
		Mustek_SendData(chip,ES01_0B_AD9826OffsetRedN,		chip->AD.OffsetR );
	else
		Mustek_SendData(chip,ES01_0A_AD9826OffsetRedP,		chip->AD.OffsetR );

	if(chip->AD.DirectionG)
		Mustek_SendData(chip,ES01_0D_AD9826OffsetGreenN,		chip->AD.OffsetG );
	else
		Mustek_SendData(chip,ES01_0C_AD9826OffsetGreenP,		chip->AD.OffsetG );

	if(chip->AD.DirectionB)
		Mustek_SendData(chip,ES01_0F_AD9826OffsetBlueN,		chip->AD.OffsetB );
	else
		Mustek_SendData(chip,ES01_0E_AD9826OffsetBlueP,		chip->AD.OffsetB );

	
	LLFSetRamAddress(chip,0x0, PackAreaStartAddress -(512*8-1), ACCESS_DRAM); 

	Mustek_SendData(chip,ES01_F3_ActionOption,
		            MOTOR_MOVE_TO_FIRST_LINE_DISABLE |
					MOTOR_BACK_HOME_AFTER_SCAN_DISABLE |
					SCAN_ENABLE |
					SCAN_BACK_TRACKING_DISABLE| 
					INVERT_MOTOR_DIRECTION_DISABLE | 
					UNIFORM_MOTOR_AND_SCAN_SPEED_ENABLE |
					ES01_STATIC_SCAN_DISABLE |
					MOTOR_TEST_LOOP_DISABLE );

	Mustek_SendData(chip,ES01_9A_AFEControl, AD9826_AFE | AUTO_CHANGE_AFE_GAIN_OFFSET_DISABLE );

    Mustek_SendData(chip,ES01_00_ADAFEConfiguration,	0x70);
    Mustek_SendData(chip,ES01_02_ADAFEMuxConfig,		0x80);

	DBG (DBG_ASIC, "SetAFEGainOffset:Exit\n");
	return status;
}

STATUS SetLEDTime(PAsic chip)
{
	STATUS status = STATUS_GOOD;
	DBG (DBG_ASIC, "SetLEDTime:Enter\n");

	Mustek_SendData (chip,ES01_B8_ChannelRedExpStartPixelLSB, LOBYTE(chip->Timing.ChannelR_StartPixel) );
	Mustek_SendData (chip,ES01_B9_ChannelRedExpStartPixelMSB, HIBYTE(chip->Timing.ChannelR_StartPixel) );
	Mustek_SendData (chip,ES01_BA_ChannelRedExpEndPixelLSB, LOBYTE(chip->Timing.ChannelR_EndPixel) );
	Mustek_SendData (chip,ES01_BB_ChannelRedExpEndPixelMSB, HIBYTE(chip->Timing.ChannelR_EndPixel) );
	
	Mustek_SendData (chip,ES01_BC_ChannelGreenExpStartPixelLSB, LOBYTE(chip->Timing.ChannelG_StartPixel) );
	Mustek_SendData (chip,ES01_BD_ChannelGreenExpStartPixelMSB, HIBYTE(chip->Timing.ChannelG_StartPixel) );
	Mustek_SendData (chip,ES01_BE_ChannelGreenExpEndPixelLSB, LOBYTE(chip->Timing.ChannelG_EndPixel) );
	Mustek_SendData (chip,ES01_BF_ChannelGreenExpEndPixelMSB, HIBYTE(chip->Timing.ChannelG_EndPixel) );
	
	Mustek_SendData (chip,ES01_C0_ChannelBlueExpStartPixelLSB, LOBYTE(chip->Timing.ChannelB_StartPixel) );
	Mustek_SendData (chip,ES01_C1_ChannelBlueExpStartPixelMSB, HIBYTE(chip->Timing.ChannelB_StartPixel) );
	Mustek_SendData (chip,ES01_C2_ChannelBlueExpEndPixelLSB, LOBYTE(chip->Timing.ChannelB_EndPixel) );
	Mustek_SendData (chip,ES01_C3_ChannelBlueExpEndPixelMSB, HIBYTE(chip->Timing.ChannelB_EndPixel) );

	DBG (DBG_ASIC, "SetLEDTime:Exit\n");
	return status;
}

STATUS SetScanMode(PAsic chip,BYTE bScanBits)
{
	STATUS status = STATUS_GOOD;
	DBG (DBG_ASIC, "SetScanMode():Enter; set f5 register\n");

	BYTE temp_f5_register = 0;
	
	if(bScanBits >= 24)
	{
		temp_f5_register |= COLOR_ES02;
	}
	else
	{
		temp_f5_register |= GRAY_ES02;
	}
	
	if( (bScanBits == 8) || (bScanBits == 24) )
	{
		temp_f5_register |= _8_BITS_ES02;
	}
	else if (bScanBits == 1)
	{
		temp_f5_register |= _1_BIT_ES02;
	}
	else
	{
		temp_f5_register |= _16_BITS_ES02;
	}
	
	BYTE GrayBWChannel;
	if(bScanBits < 24)
	{
		GrayBWChannel = 1;
	}
	else
	{
		GrayBWChannel = 4;
	}
	
	if (GrayBWChannel == 0)
	{
		temp_f5_register |= GRAY_RED_ES02;
	}
	else if (GrayBWChannel == 1)
	{
		temp_f5_register |= GRAY_GREEN_ES02;
	}
	else if (GrayBWChannel == 2)
	{
		temp_f5_register |= GRAY_BLUE_ES02;
	}
	else
	{
		temp_f5_register |= GRAY_GREEN_BLUE_ES02;
	}
	
	status=Mustek_SendData(chip,ES01_F5_ScanDataFormat, temp_f5_register);

	DBG (DBG_ASIC, "F5_ScanDataFormat=0x%x\n",temp_f5_register);
	DBG (DBG_ASIC, "SetScanMode():Exit\n");
	return status;	
}

STATUS SetPackAddress(PAsic chip,WORD wXResolution, WORD wWidth,WORD wX,
					  double XRatioAdderDouble,double XRatioTypeDouble,
					  BYTE byClear_Pulse_Width, WORD *PValidPixelNumber)
{
	STATUS status = STATUS_GOOD;
	DBG (DBG_ASIC, "SetPackAddress:Enter\n");

	WORD LineTotalOverlapPixel;
	BYTE OverLapPixel;
	BYTE TotalLineShift;
	BYTE InvalidPixelNumberBackup;
	WORD SegmentTotalPixel;
	DWORD dwLineTotalPixel;
	WORD ValidPixelNumber =*PValidPixelNumber;

	
	DWORD FinalLinePixelPerSegment;
	BYTE InValidPixelNumber;
	DWORD CISPackAreaStartAddress;
	BYTE PackAreaUseLine;

	LineTotalOverlapPixel = 0;
	OverLapPixel = 0;
	TotalLineShift = 1;  
	PackAreaUseLine = TotalLineShift + 1;

	if (wXResolution > (SENSOR_DPI/2))
	{
		ValidPixelNumber = ValidPixelNumberFor1200DPI;
		OverLapPixel = OverLapPixelNumber1200;
	}
	else
	{
		ValidPixelNumber = ValidPixelNumberFor600DPI;
		OverLapPixel = OverLapPixelNumber600;
	}

	ValidPixelNumber = (WORD) ( (wWidth + 10 + 15)*XRatioAdderDouble );
	ValidPixelNumber >>= 4;
	ValidPixelNumber <<= 4;

	ValidPixelNumber += (OverLapPixel*2);

	int i;
	for (i=0 ; i<16 ; i++)	{
		Mustek_SendData(chip,ES01_2B0_SEGMENT0_OVERLAP_SEGMENT1 + i, OverLapPixel);
		Mustek_SendData(chip,ES01_2C0_VALID_PIXEL_PARAMETER_OF_SEGMENT1 + i, 0);
	}
	LineTotalOverlapPixel = OverLapPixel*16;

	FinalLinePixelPerSegment = ValidPixelNumber + OverLapPixel*2;

	if ((FinalLinePixelPerSegment%8) > 0){
		InValidPixelNumber = (BYTE) (8 - (FinalLinePixelPerSegment%8));
	}else{
		InValidPixelNumber = 0;
	}

	InvalidPixelNumberBackup = InValidPixelNumber;
	
	Mustek_SendData(chip,ES01_1B0_SEGMENT_PIXEL_NUMBER_LB, LOBYTE(ValidPixelNumber)); 
	Mustek_SendData(chip,ES01_1B1_SEGMENT_PIXEL_NUMBER_HB, HIBYTE(ValidPixelNumber));

	SegmentTotalPixel = ValidPixelNumber + OverLapPixel*2 + InValidPixelNumber;

	Mustek_SendData(chip,ES01_169_NUMBER_OF_SEGMENT_PIXEL_LB, LOBYTE(ValidPixelNumber));
	Mustek_SendData(chip,ES01_16A_NUMBER_OF_SEGMENT_PIXEL_HB, HIBYTE(ValidPixelNumber));
	Mustek_SendData(chip,ES01_16B_BETWEEN_SEGMENT_INVALID_PIXEL, 0);

	Mustek_SendData(chip,ES01_B6_LineWidthPixelLSB, LOBYTE (ValidPixelNumber));
	Mustek_SendData(chip,ES01_B7_LineWidthPixelMSB, HIBYTE (ValidPixelNumber));

	Mustek_SendData(chip,ES01_19A_CHANNEL_LINE_GAP_LB, LOBYTE (ValidPixelNumber));
	Mustek_SendData(chip,ES01_19B_CHANNEL_LINE_GAP_HB, HIBYTE (ValidPixelNumber));
	DBG (DBG_ASIC, "ValidPixelNumber=%d\n",ValidPixelNumber);

	for (i=0 ; i<36 ; i++){
		Mustek_SendData(chip,0x270 + i, 0 );
	}

	Mustek_SendData(chip,0x270, (BYTE) ((SegmentTotalPixel*(PackAreaUseLine)*1)) );
	Mustek_SendData(chip,0x271, (BYTE) ((SegmentTotalPixel*(PackAreaUseLine)*1)>>8) );
	Mustek_SendData(chip,0x272, (BYTE) ((SegmentTotalPixel*(PackAreaUseLine)*1)>>16) );

	Mustek_SendData(chip,0x27C, (BYTE) ((SegmentTotalPixel*(PackAreaUseLine)*2)) );
	Mustek_SendData(chip,0x27D, (BYTE) ((SegmentTotalPixel*(PackAreaUseLine)*2)>>8) );
	Mustek_SendData(chip,0x27E, (BYTE) ((SegmentTotalPixel*(PackAreaUseLine)*2)>>16) );

	Mustek_SendData(chip,0x288, (BYTE) ((SegmentTotalPixel*(PackAreaUseLine)*3)) );
	Mustek_SendData(chip,0x289, (BYTE) ((SegmentTotalPixel*(PackAreaUseLine)*3)>>8) );
	Mustek_SendData(chip,0x28A, (BYTE) ((SegmentTotalPixel*(PackAreaUseLine)*3)>>16) );
	DBG (DBG_ASIC, "channel gap=%d\n",SegmentTotalPixel*(PackAreaUseLine));


	Mustek_SendData(chip,ES01_B4_StartPixelLSB, LOBYTE (wX + 0));
	Mustek_SendData(chip,ES01_B5_StartPixelMSB, HIBYTE (wX + 0));


	dwLineTotalPixel = ValidPixelNumber;

	Mustek_SendData(chip,ES01_1B9_LINE_PIXEL_NUMBER_LB, LOBYTE ( XRatioTypeDouble*(dwLineTotalPixel-1) ) );
	Mustek_SendData(chip,ES01_1BA_LINE_PIXEL_NUMBER_HB, HIBYTE ( XRatioTypeDouble*(dwLineTotalPixel-1) ) );

	/* final start read out pixel */
	Mustek_SendData(chip,ES01_1F4_START_READ_OUT_PIXEL_LB, LOBYTE (0) );
	Mustek_SendData(chip,ES01_1F5_START_READ_OUT_PIXEL_HB, HIBYTE (0) );

	DWORD MaxPixelHW ;
	MaxPixelHW = (dwLineTotalPixel + InValidPixelNumber) - 10;

	if ( wWidth > MaxPixelHW)
	{
		fprintf(stderr, "read out pixel over max pixel! image will shift!!!\n");
	}

	/* final read pixel width */
	Mustek_SendData(chip,ES01_1F6_READ_OUT_PIXEL_LENGTH_LB, LOBYTE ( wWidth + 9 ) );
	Mustek_SendData(chip,ES01_1F7_READ_OUT_PIXEL_LENGTH_HB, HIBYTE ( wWidth + 9 ) );

	/* data output sequence */
	Mustek_SendData(chip,ES01_1F8_PACK_CHANNEL_SELECT_B0, 0 );
	Mustek_SendData(chip,ES01_1F9_PACK_CHANNEL_SELECT_B1, 0 );
	Mustek_SendData(chip,ES01_1FA_PACK_CHANNEL_SELECT_B2, 0x18 ); 

	Mustek_SendData(chip,ES01_1FB_PACK_CHANNEL_SIZE_B0, (BYTE) ((SegmentTotalPixel*PackAreaUseLine)) );
	Mustek_SendData(chip,ES01_1FC_PACK_CHANNEL_SIZE_B1, (BYTE) ((SegmentTotalPixel*PackAreaUseLine)>>8) );
	Mustek_SendData(chip,ES01_1FD_PACK_CHANNEL_SIZE_B2, (BYTE) ((SegmentTotalPixel*PackAreaUseLine)>>16) );

	Mustek_SendData(chip,ES01_16C_LINE_SHIFT_OUT_TIMES_DIRECTION, 0x01);
	Mustek_SendData(chip,ES01_1CE_LINE_SEGMENT_NUMBER, 0x00);
	Mustek_SendData(chip,ES01_D8_PHTG_EDGE_TIMING_ADJUST, 0x17);

	Mustek_SendData(chip,ES01_D9_CLEAR_PULSE_WIDTH, byClear_Pulse_Width);

	Mustek_SendData(chip,ES01_DA_CLEAR_SIGNAL_INVERTING_OUTPUT, 0x54 | 0x01);
	Mustek_SendData(chip,ES01_CD_TG_R_CONTROL, 0x3C);
	Mustek_SendData(chip,ES01_CE_TG_G_CONTROL, 0);
	Mustek_SendData(chip,ES01_CF_TG_B_CONTROL, 0x3C);


	/* set pack area address */

	CISPackAreaStartAddress = PackAreaStartAddress;
	DBG (DBG_ASIC, "CISPackAreaStartAddress=%d\n",CISPackAreaStartAddress);
	
	/* cycle 1 */
	Mustek_SendData(chip,ES01_16D_EXPOSURE_CYCLE1_SEGMENT1_START_ADDR_BYTE0, (BYTE) ((CISPackAreaStartAddress + 0)) );
	Mustek_SendData(chip,ES01_16E_EXPOSURE_CYCLE1_SEGMENT1_START_ADDR_BYTE1, (BYTE) ((CISPackAreaStartAddress + 0)>>8) );
	Mustek_SendData(chip,ES01_16F_EXPOSURE_CYCLE1_SEGMENT1_START_ADDR_BYTE2, (BYTE) ((CISPackAreaStartAddress + 0)>>16) );

	Mustek_SendData(chip,ES01_170_EXPOSURE_CYCLE1_SEGMENT2_START_ADDR_BYTE0, (BYTE) ((CISPackAreaStartAddress + 0xC0000)) );
	Mustek_SendData(chip,ES01_171_EXPOSURE_CYCLE1_SEGMENT2_START_ADDR_BYTE1, (BYTE) ((CISPackAreaStartAddress + 0xC0000)>>8) );
	Mustek_SendData(chip,ES01_172_EXPOSURE_CYCLE1_SEGMENT2_START_ADDR_BYTE2, (BYTE) ((CISPackAreaStartAddress + 0xC0000)>>16) );

	Mustek_SendData(chip,ES01_173_EXPOSURE_CYCLE1_SEGMENT3_START_ADDR_BYTE0, (BYTE) ((CISPackAreaStartAddress + 0xC0000)) );
	Mustek_SendData(chip,ES01_174_EXPOSURE_CYCLE1_SEGMENT3_START_ADDR_BYTE1, (BYTE) ((CISPackAreaStartAddress + 0xC0000)>>8) );
	Mustek_SendData(chip,ES01_175_EXPOSURE_CYCLE1_SEGMENT3_START_ADDR_BYTE2, (BYTE) ((CISPackAreaStartAddress + 0xC0000)>>16) );

	Mustek_SendData(chip,ES01_176_EXPOSURE_CYCLE1_SEGMENT4_START_ADDR_BYTE0, (BYTE) ((CISPackAreaStartAddress + 0xC0000)) );
	Mustek_SendData(chip,ES01_177_EXPOSURE_CYCLE1_SEGMENT4_START_ADDR_BYTE1, (BYTE) ((CISPackAreaStartAddress + 0xC0000)>>8) );
	Mustek_SendData(chip,ES01_178_EXPOSURE_CYCLE1_SEGMENT4_START_ADDR_BYTE2, (BYTE) ((CISPackAreaStartAddress + 0xC0000)>>16) );

	/* cycle 2 */
	Mustek_SendData(chip,ES01_179_EXPOSURE_CYCLE2_SEGMENT1_START_ADDR_BYTE0, (BYTE) ((CISPackAreaStartAddress + 0xC0000)) );
	Mustek_SendData(chip,ES01_17A_EXPOSURE_CYCLE2_SEGMENT1_START_ADDR_BYTE1, (BYTE) ((CISPackAreaStartAddress + 0xC0000)>>8) );
	Mustek_SendData(chip,ES01_17B_EXPOSURE_CYCLE2_SEGMENT1_START_ADDR_BYTE2, (BYTE) ((CISPackAreaStartAddress + 0xC0000)>>16) );

	Mustek_SendData(chip,ES01_17C_EXPOSURE_CYCLE2_SEGMENT2_START_ADDR_BYTE0, (BYTE) ((CISPackAreaStartAddress + 0xC0000)) );
	Mustek_SendData(chip,ES01_17D_EXPOSURE_CYCLE2_SEGMENT2_START_ADDR_BYTE1, (BYTE) ((CISPackAreaStartAddress + 0xC0000)>>8) );
	Mustek_SendData(chip,ES01_17E_EXPOSURE_CYCLE2_SEGMENT2_START_ADDR_BYTE2, (BYTE) ((CISPackAreaStartAddress + 0xC0000)>>16) );

	Mustek_SendData(chip,ES01_17F_EXPOSURE_CYCLE2_SEGMENT3_START_ADDR_BYTE0, (BYTE) ((CISPackAreaStartAddress + 0xC0000)) );
	Mustek_SendData(chip,ES01_180_EXPOSURE_CYCLE2_SEGMENT3_START_ADDR_BYTE1, (BYTE) ((CISPackAreaStartAddress + 0xC0000)>>8) );
	Mustek_SendData(chip,ES01_181_EXPOSURE_CYCLE2_SEGMENT3_START_ADDR_BYTE2, (BYTE) ((CISPackAreaStartAddress + 0xC0000)>>16) );

	Mustek_SendData(chip,ES01_182_EXPOSURE_CYCLE2_SEGMENT4_START_ADDR_BYTE0, (BYTE) ((CISPackAreaStartAddress + 0xC0000)) );
	Mustek_SendData(chip,ES01_183_EXPOSURE_CYCLE2_SEGMENT4_START_ADDR_BYTE1, (BYTE) ((CISPackAreaStartAddress + 0xC0000)>>8) );
	Mustek_SendData(chip,ES01_184_EXPOSURE_CYCLE2_SEGMENT4_START_ADDR_BYTE2, (BYTE) ((CISPackAreaStartAddress + 0xC0000)>>16) );

	/* cycle 3 */
	Mustek_SendData(chip,ES01_185_EXPOSURE_CYCLE3_SEGMENT1_START_ADDR_BYTE0, (BYTE) ((CISPackAreaStartAddress + 0xC0000)) );

	Mustek_SendData(chip,ES01_186_EXPOSURE_CYCLE3_SEGMENT1_START_ADDR_BYTE1, (BYTE) ((CISPackAreaStartAddress + 0xC0000)>>8) );
	Mustek_SendData(chip,ES01_187_EXPOSURE_CYCLE3_SEGMENT1_START_ADDR_BYTE2, (BYTE) ((CISPackAreaStartAddress + 0xC0000)>>16) );

	Mustek_SendData(chip,ES01_188_EXPOSURE_CYCLE3_SEGMENT2_START_ADDR_BYTE0, (BYTE) ((CISPackAreaStartAddress + 0xC0000)) );
	Mustek_SendData(chip,ES01_189_EXPOSURE_CYCLE3_SEGMENT2_START_ADDR_BYTE1, (BYTE) ((CISPackAreaStartAddress + 0xC0000)>>8) );
	Mustek_SendData(chip,ES01_18A_EXPOSURE_CYCLE3_SEGMENT2_START_ADDR_BYTE2, (BYTE) ((CISPackAreaStartAddress + 0xC0000)>>16) );

	Mustek_SendData(chip,ES01_18B_EXPOSURE_CYCLE3_SEGMENT3_START_ADDR_BYTE0, (BYTE) ((CISPackAreaStartAddress + 0xC0000)) );
	Mustek_SendData(chip,ES01_18C_EXPOSURE_CYCLE3_SEGMENT3_START_ADDR_BYTE1, (BYTE) ((CISPackAreaStartAddress + 0xC0000)>>8) );
	Mustek_SendData(chip,ES01_18D_EXPOSURE_CYCLE3_SEGMENT3_START_ADDR_BYTE2, (BYTE) ((CISPackAreaStartAddress + 0xC0000)>>16) );

	Mustek_SendData(chip,ES01_18E_EXPOSURE_CYCLE3_SEGMENT4_START_ADDR_BYTE0, (BYTE) ((CISPackAreaStartAddress + 0xC0000)) );
	Mustek_SendData(chip,ES01_18F_EXPOSURE_CYCLE3_SEGMENT4_START_ADDR_BYTE1, (BYTE) ((CISPackAreaStartAddress + 0xC0000)>>8) );
	Mustek_SendData(chip,ES01_190_EXPOSURE_CYCLE3_SEGMENT4_START_ADDR_BYTE2, (BYTE) ((CISPackAreaStartAddress + 0xC0000)>>16) );
	DBG (DBG_ASIC, "set CISPackAreaStartAddress ok\n");
	
	Mustek_SendData(chip,0x260, InValidPixelNumber );
	Mustek_SendData(chip,0x261, InValidPixelNumber<<4 );
	Mustek_SendData(chip,0x262, InValidPixelNumber );
	Mustek_SendData(chip,0x263, 0 );
	DBG (DBG_ASIC, "InValidPixelNumber=%d\n",InValidPixelNumber);

	Mustek_SendData(chip,0x264, 0 );
	Mustek_SendData(chip,0x265, 0 );
	Mustek_SendData(chip,0x266, 0 );
	Mustek_SendData(chip,0x267, 0 );

	Mustek_SendData(chip,0x268, 0 );
	Mustek_SendData(chip,0x269, 0 );
	Mustek_SendData(chip,0x26A, 0 );
	Mustek_SendData(chip,0x26B, 0 );

	Mustek_SendData(chip,0x26C, 0 );
	Mustek_SendData(chip,0x26D, 0 );
	Mustek_SendData(chip,0x26E, 0 );
	Mustek_SendData(chip,0x26F, 0 );
	DBG (DBG_ASIC, "Set Invalid Pixel ok\n");
	

	/* Pack Start Address */
	Mustek_SendData(chip,ES01_19E_PACK_AREA_R_START_ADDR_BYTE0, (BYTE) ((CISPackAreaStartAddress + (SegmentTotalPixel*(PackAreaUseLine*0)))) );
	Mustek_SendData(chip,ES01_19F_PACK_AREA_R_START_ADDR_BYTE1, (BYTE) ((CISPackAreaStartAddress + (SegmentTotalPixel*(PackAreaUseLine*0)))>>8) );
	Mustek_SendData(chip,ES01_1A0_PACK_AREA_R_START_ADDR_BYTE2, (BYTE) ((CISPackAreaStartAddress + (SegmentTotalPixel*(PackAreaUseLine*0)))>>16) );


	Mustek_SendData(chip,ES01_1A1_PACK_AREA_G_START_ADDR_BYTE0, (BYTE) ((CISPackAreaStartAddress + (SegmentTotalPixel*(PackAreaUseLine*1)))) );
	Mustek_SendData(chip,ES01_1A2_PACK_AREA_G_START_ADDR_BYTE1, (BYTE) ((CISPackAreaStartAddress + (SegmentTotalPixel*(PackAreaUseLine*1)))>>8) );
	Mustek_SendData(chip,ES01_1A3_PACK_AREA_G_START_ADDR_BYTE2, (BYTE) ((CISPackAreaStartAddress + (SegmentTotalPixel*(PackAreaUseLine*1)))>>16) );

	Mustek_SendData(chip,ES01_1A4_PACK_AREA_B_START_ADDR_BYTE0, (BYTE) ((CISPackAreaStartAddress + (SegmentTotalPixel*(PackAreaUseLine*2)))) );
	Mustek_SendData(chip,ES01_1A5_PACK_AREA_B_START_ADDR_BYTE1, (BYTE) ((CISPackAreaStartAddress + (SegmentTotalPixel*(PackAreaUseLine*2)))>>8) );
	Mustek_SendData(chip,ES01_1A6_PACK_AREA_B_START_ADDR_BYTE2, (BYTE) ((CISPackAreaStartAddress + (SegmentTotalPixel*(PackAreaUseLine*2)))>>16) );

	/* Pack End Address */
	Mustek_SendData(chip,ES01_1A7_PACK_AREA_R_END_ADDR_BYTE0, (BYTE) ((CISPackAreaStartAddress + (SegmentTotalPixel*(PackAreaUseLine*1)-1))) );
	Mustek_SendData(chip,ES01_1A8_PACK_AREA_R_END_ADDR_BYTE1, (BYTE) ((CISPackAreaStartAddress + (SegmentTotalPixel*(PackAreaUseLine*1)-1))>>8) );
	Mustek_SendData(chip,ES01_1A9_PACK_AREA_R_END_ADDR_BYTE2, (BYTE) ((CISPackAreaStartAddress + (SegmentTotalPixel*(PackAreaUseLine*1)-1))>>16) );

	Mustek_SendData(chip,ES01_1AA_PACK_AREA_G_END_ADDR_BYTE0, (BYTE) ((CISPackAreaStartAddress + (SegmentTotalPixel*(PackAreaUseLine*2)-1))) );
	Mustek_SendData(chip,ES01_1AB_PACK_AREA_G_END_ADDR_BYTE1, (BYTE) ((CISPackAreaStartAddress + (SegmentTotalPixel*(PackAreaUseLine*2)-1))>>8) );
	Mustek_SendData(chip,ES01_1AC_PACK_AREA_G_END_ADDR_BYTE2, (BYTE) ((CISPackAreaStartAddress + (SegmentTotalPixel*(PackAreaUseLine*2)-1))>>16) );

	Mustek_SendData(chip,ES01_1AD_PACK_AREA_B_END_ADDR_BYTE0, (BYTE) ((CISPackAreaStartAddress + (SegmentTotalPixel*(PackAreaUseLine*3)-1))) );
	Mustek_SendData(chip,ES01_1AE_PACK_AREA_B_END_ADDR_BYTE1, (BYTE) ((CISPackAreaStartAddress + (SegmentTotalPixel*(PackAreaUseLine*3)-1))>>8) );
	Mustek_SendData(chip,ES01_1AF_PACK_AREA_B_END_ADDR_BYTE2, (BYTE) ((CISPackAreaStartAddress + (SegmentTotalPixel*(PackAreaUseLine*3)-1))>>16) );
	DBG (DBG_ASIC, "CISPackAreaStartAddress + (SegmentTotalPixel*(PackAreaUseLine*1))=%d\n",(CISPackAreaStartAddress + (SegmentTotalPixel*(PackAreaUseLine*1))));
	
	Mustek_SendData(chip,ES01_19C_MAX_PACK_LINE, PackAreaUseLine );
	
	status=Mustek_SendData(chip,ES01_19D_PACK_THRESHOLD_LINE, TotalLineShift );
	DBG (DBG_ASIC, "PackAreaUseLine=%d,TotalLineShift=%d\n",PackAreaUseLine,TotalLineShift);

	*PValidPixelNumber=ValidPixelNumber;
	
	DBG (DBG_ASIC, "SetPackAddress:Enter\n");
	return status;	
}

STATUS SetExtraSetting(PAsic chip,WORD wXResolution,WORD  wCCD_PixelNumber,BOOL isCaribrate)
{
	STATUS status = STATUS_GOOD;
	DBG (DBG_ASIC, "SetExtraSetting:Enter\n");
	
	Mustek_SendData (chip,ES01_B8_ChannelRedExpStartPixelLSB, LOBYTE(chip->Timing.ChannelR_StartPixel) );
	Mustek_SendData (chip,ES01_B9_ChannelRedExpStartPixelMSB, HIBYTE(chip->Timing.ChannelR_StartPixel) );
	Mustek_SendData (chip,ES01_BA_ChannelRedExpEndPixelLSB, LOBYTE(chip->Timing.ChannelR_EndPixel) );
	Mustek_SendData (chip,ES01_BB_ChannelRedExpEndPixelMSB, HIBYTE(chip->Timing.ChannelR_EndPixel) );
	
	Mustek_SendData (chip,ES01_BC_ChannelGreenExpStartPixelLSB, LOBYTE(chip->Timing.ChannelG_StartPixel) );
	Mustek_SendData (chip,ES01_BD_ChannelGreenExpStartPixelMSB, HIBYTE(chip->Timing.ChannelG_StartPixel) );
	Mustek_SendData (chip,ES01_BE_ChannelGreenExpEndPixelLSB, LOBYTE(chip->Timing.ChannelG_EndPixel) );
	Mustek_SendData (chip,ES01_BF_ChannelGreenExpEndPixelMSB, HIBYTE(chip->Timing.ChannelG_EndPixel) );
	
	Mustek_SendData (chip,ES01_C0_ChannelBlueExpStartPixelLSB, LOBYTE(chip->Timing.ChannelB_StartPixel) );
	Mustek_SendData (chip,ES01_C1_ChannelBlueExpStartPixelMSB, HIBYTE(chip->Timing.ChannelB_StartPixel) );
	Mustek_SendData (chip,ES01_C2_ChannelBlueExpEndPixelLSB, LOBYTE(chip->Timing.ChannelB_EndPixel) );
	Mustek_SendData (chip,ES01_C3_ChannelBlueExpEndPixelMSB, HIBYTE(chip->Timing.ChannelB_EndPixel) );
	
	BYTE	byPHTG_PulseWidth, byPHTG_WaitWidth;
	byPHTG_PulseWidth = chip->Timing.PHTG_PluseWidth;
	byPHTG_WaitWidth = chip->Timing.PHTG_WaitWidth;
	Mustek_SendData (chip,ES01_B2_PHTGPulseWidth, byPHTG_PulseWidth );
	Mustek_SendData (chip,ES01_B3_PHTGWaitWidth, byPHTG_WaitWidth );
	
	Mustek_SendData (chip,ES01_CC_PHTGTimingAdjust, chip->Timing.PHTG_TimingAdj);	
	Mustek_SendData (chip,ES01_D0_PH1_0, chip->Timing.PHTG_TimingSetup );
	
	DBG (DBG_ASIC, "ChannelR_StartPixel=%d,ChannelR_EndPixel=%d\n",chip->Timing.ChannelR_StartPixel,chip->Timing.ChannelR_EndPixel);
	
	if (wXResolution == 1200)
	{
		Mustek_SendData (chip,ES01_DE_CCD_SETUP_REGISTER, chip->Timing.DE_CCD_SETUP_REGISTER_1200);
	}
	else
	{
		Mustek_SendData (chip,ES01_DE_CCD_SETUP_REGISTER, chip->Timing.DE_CCD_SETUP_REGISTER_600); 
	}
	
	BYTE temp_ff_register = 0;
	
	if(isCaribrate==TRUE)
	{
		temp_ff_register |= BYPASS_DARK_SHADING_ENABLE;
		temp_ff_register |= BYPASS_WHITE_SHADING_ENABLE;
	}
	else/*Setwindow*/
	{
		temp_ff_register |= BYPASS_DARK_SHADING_DISABLE;
		temp_ff_register |= BYPASS_WHITE_SHADING_DISABLE;
	}
	
	temp_ff_register |= BYPASS_PRE_GAMMA_ENABLE;
	
	temp_ff_register |= BYPASS_CONVOLUTION_ENABLE;
	
	
	temp_ff_register |= BYPASS_MATRIX_ENABLE;
	
	temp_ff_register |= BYPASS_GAMMA_ENABLE;
	
	if(isCaribrate==TRUE)
	{
		Mustek_SendData (chip,ES01_FF_SCAN_IMAGE_OPTION, 0xfc | (0x00 & 0x03) );
		DBG (DBG_ASIC, "FF_SCAN_IMAGE_OPTION=0x%x\n",0xfc | (0x00 & 0x03));
	}
	else/*Setwindow*/
	{
		Mustek_SendData (chip,ES01_FF_SCAN_IMAGE_OPTION, temp_ff_register | (0x00 & 0x03) );
		DBG (DBG_ASIC, "FF_SCAN_IMAGE_OPTION=0x%x\n", temp_ff_register| (0x00 & 0x03));
	}
	
	/*  pixel process time */
	Mustek_SendData (chip,ES01_B0_CCDPixelLSB, LOBYTE (wCCD_PixelNumber) );
	Mustek_SendData (chip,ES01_B1_CCDPixelMSB, HIBYTE (wCCD_PixelNumber) );
	Mustek_SendData (chip,ES01_DF_ICG_CONTROL, 0x17);
	DBG (DBG_ASIC, "wCCD_PixelNumber=%d\n",wCCD_PixelNumber);
	
	BYTE bThreshold = 128;

	Mustek_SendData(chip,ES01_88_LINE_ART_THRESHOLD_HIGH_VALUE, bThreshold);
	Mustek_SendData(chip,ES01_89_LINE_ART_THRESHOLD_LOW_VALUE, bThreshold-1);
	DBG (DBG_ASIC, "bThreshold=%d\n",bThreshold);

	usleep (50000);

	DBG (DBG_ASIC, "SetExtraSetting:Exit\n");
	return status;	
}
