/*Asic_motor.h: function of set motor parameter and action;*/
/* sane - Scanner Access Now Easy.

   Copyright (C) 2005 Mustek.
   Originally maintained by Mustek
   Author:Roy 2005.5.24

   This file is part of the SANE package.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston,
   MA 02111-1307, USA.

   As a special exception, the authors of SANE give permission for
   additional uses of the libraries contained in this release of SANE.

   The exception is that, if you link a SANE library with other files
   to produce an executable, this does not by itself cause the
   resulting executable to be covered by the GNU General Public
   License.  Your use of that executable is in no way restricted on
   account of linking the SANE library code into it.

   This exception does not, however, invalidate any other reasons why
   the executable file might be covered by the GNU General Public
   License.

   If you submit changes to SANE to the maintainers to be included in
   a subsequent release, you agree by submitting the changes that
   those changes may be distributed with this exception intact.

   If you write modifications of your own for SANE, it is your choice
   whether to permit this exception to apply to your modifications.
   If you do not wish that, delete this exception notice.

   This file implements a SANE backend for Mustek 1200UB and similar 
   USB flatbed scanners.  */
#ifndef _ASIC_MOTOR_H_
#define _ASIC_MOTOR_H_

#define ACTION_MODE_ACCDEC_MOVE 0
#define ACTION_MODE_UNIFORM_SPEED_MOVE 1


typedef struct tagMOTOR_CURRENT_AND_PHASE
{
	BYTE MoveType;
	BYTE FillPhase;
	BYTE MotorDriverIs3967;
	BYTE MotorCurrentTableA[32];
	BYTE MotorCurrentTableB[32];
	BYTE MotorPhaseTable[32];
}MOTOR_CURRENT_AND_PHASE, LPMOTOR_CURRENT_AND_PHASE;

typedef struct tagLLF_RAMACCESS
{
	BYTE ReadWrite;
	BYTE IsOnChipGamma;
	WORD LoStartAddress;
	WORD HiStartAddress;
	int RwSize;
	BYTE DramDelayTime;
	BYTE *BufferPtr;
}LLF_RAMACCESS;

typedef struct tagLLF_MOTOR_CURRENT_AND_PHASE
{
	BYTE MoveType;
	BYTE FillPhase;
	BYTE MotorDriverIs3967;
	BYTE MotorCurrentTableA[32];
	BYTE MotorCurrentTableB[32];
	BYTE MotorPhaseTable[32];
}LLF_MOTOR_CURRENT_AND_PHASE;

typedef struct tagLLF_CALCULATEMOTORTABLE
{
	WORD StartSpeed;
	WORD EndSpeed;
	WORD AccStepBeforeScan;
	BYTE DecStepAfterScan;
	LPWORD lpMotorTable;
}LLF_CALCULATEMOTORTABLE;

typedef struct tagLLF_SETMOTORTABLE
{
	DWORD TableSize;
	BYTE MotorTableAddress;
	WORD *MotorTablePtr;
}LLF_SETMOTORTABLE;

typedef struct tagLLF_MOTORMOVE
{
	BYTE ActionMode; /* 0: AccDec Mode, 1: Uniform Speed Mode, 2: Test Mode */
	BYTE ActionType; /* 0: Forward, 1: Backward, 2:Back To Home */
	BYTE MotorSelect; /* 0: Motor 0 only, 1: Motor 1 only, 2: Motor 0 & 1 */
	BYTE HomeSensorSelect; /* 0: Sensor 0, 1: Sensor 1, 2: Sensor 0 & 1, 3:Invert Sensor 1 */
	WORD FixMoveSpeed;
	DWORD FixMoveSteps; /* 3 bytes */
	BYTE MotorSpeedUnit;
	BYTE MotorSyncUnit;
	WORD AccStep; /*Max = 511 */
	BYTE DecStep; /* Max = 255 */
	BYTE MotorMoveUnit;
	BYTE WaitOrNoWait; /* 0: no wait, 1: wait */
	BYTE Lamp0PwmFreq; /* Lamp0 PWM freq */
	BYTE Lamp1PwmFreq; /* Lamp1 PWM freq */
	
	WORD wForwardSteps;
	WORD wScanAccSteps;
	BYTE bScanDecSteps;
	WORD wFixScanSteps;
	WORD wScanBackTrackingSteps;
	WORD wScanRestartSteps;
	WORD wScanBackHomeExtSteps;
}LLF_MOTORMOVE;

extern STATUS CalculateMotorTable(LLF_CALCULATEMOTORTABLE *lpCalculateMotorTable, WORD wYResolution);
extern STATUS LLFCalculateMotorTable(LLF_CALCULATEMOTORTABLE *lpCalculateMotorTable);

extern STATUS LLFSetMotorCurrentAndPhase (PAsic chip, LLF_MOTOR_CURRENT_AND_PHASE *MotorCurrentAndPhase);
extern STATUS SetMotorStepTable(PAsic chip,LLF_MOTORMOVE *MotorStepsTable,WORD wStartY, DWORD dwScanImageSteps, WORD wYResolution);
extern STATUS LLFSetMotorTable (PAsic chip,LLF_SETMOTORTABLE *LLF_SetMotorTable);
extern STATUS SetMotorCurrent(PAsic chip,WORD dwMotorSpeed, LLF_MOTOR_CURRENT_AND_PHASE *CurrentPhase);
extern STATUS LLFMotorMove (PAsic chip,LLF_MOTORMOVE *LLF_MotorMove);
extern STATUS LLFStopMotorMove (PAsic chip);

extern STATUS LLFSetRamAddress(PAsic chip,DWORD dwStartAddr, DWORD dwEndAddr, BYTE byAccessTarget);
extern STATUS LLFRamAccess(PAsic chip,LLF_RAMACCESS *RamAccess);
extern STATUS MotorBackHome (PAsic chip,BYTE WaitOrNoWait);

#endif
