/*Asic.c: interface of spicall;*/
/* sane - Scanner Access Now Easy.

   Copyright (C) 2005 Mustek.
   Originally maintained by Mustek
   Author:Roy 2005.5.24

   This file is part of the SANE package.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston,
   MA 02111-1307, USA.

   As a special exception, the authors of SANE give permission for
   additional uses of the libraries contained in this release of SANE.

   The exception is that, if you link a SANE library with other files
   to produce an executable, this does not by itself cause the
   resulting executable to be covered by the GNU General Public
   License.  Your use of that executable is in no way restricted on
   account of linking the SANE library code into it.

   This exception does not, however, invalidate any other reasons why
   the executable file might be covered by the GNU General Public
   License.

   If you submit changes to SANE to the maintainers to be included in
   a subsequent release, you agree by submitting the changes that
   those changes may be distributed with this exception intact.

   If you write modifications of your own for SANE, it is your choice
   whether to permit this exception to apply to your modifications.
   If you do not wish that, delete this exception notice.

   This file implements a SANE backend for Mustek 1200UB and similar 
   USB flatbed scanners.  */

#include "mustek_usb2_asic_motor.c"
#include "mustek_usb2_asic_motor.c"


WORD ProductID = 0x0409;
WORD VendorID = 0x055f;

BYTE	bScanBits;

STATUS Asic_Open(PAsic chip, LPSTR pDeviceName)
{
	STATUS status;
	XDBG(DEBUG_ASIC,"Asic_Open: Enter");

	BYTE devname[MAX_PATH];

	if (chip->firmwarestate > FS_OPENED)
	{
		XDBG(DEBUG_ASIC,"chip has been opened. fd=%d",chip->fd);
		return STATUS_INVAL;
	}

	/*init usb*/
	if(STATUS_GOOD!=libusb_init(VendorID,ProductID,&chip->fd))
	{
		fprintf(stderr,"Can't find  scanner vid=0x%04x,pid=0x%04x \n",VendorID,ProductID);
		return STATUS_INVAL;
	}

	/*open usb*/
	if(STATUS_GOOD!=libusb_open(chip->fd))
	{
		fprintf(stderr,"Can't open scanner! \n");
		return STATUS_INVAL;
	}

	/*open scanner chip*/
	status = OpenScanChip(chip);
	if (status != STATUS_GOOD)
	{
		libusb_close(chip->fd);
		XDBG(DEBUG_ASIC,"Asic_Open: OpenScanChip error");
		return status;
	}
	
	Mustek_SendData(chip,ES01_94_PowerSaveControl, 0x27);
	Mustek_SendData(chip,ES01_86_DisableAllClockWhenIdle, CLOSE_ALL_CLOCK_DISABLE);
	Mustek_SendData(chip,ES01_79_AFEMCLK_SDRAMCLK_DELAY_CONTROL, SDRAMCLK_DELAY_12_ns);
	
	/* SDRAM initial sequence*/
	Mustek_SendData(chip,ES01_87_SDRAM_Timing, 0xf1);
	Mustek_SendData(chip,ES01_87_SDRAM_Timing, 0xa5);
	Mustek_SendData(chip,ES01_87_SDRAM_Timing, 0x91);
	Mustek_SendData(chip,ES01_87_SDRAM_Timing, 0x81);
	Mustek_SendData(chip,ES01_87_SDRAM_Timing, 0xf0);


	chip->firmwarestate = FS_OPENED;
	Asic_WaitUnitReady(chip);
	XDBG(DEBUG_ASIC,"Asic_WaitUnitReady");
	status = SafeInitialChip(chip);
	if (status != STATUS_GOOD)
	{
		fprintf(stderr,"Asic_Open: SafeInitialChip error\n");
		return status;
	}
	

	XDBG(DEBUG_ASIC,"Asic_Open: Exit");
	return status;
}


STATUS Asic_Close(PAsic chip)
{
	STATUS status;
	XDBG(DEBUG_ASIC,"Asic_Close: Enter");
	
	if (chip->firmwarestate < FS_OPENED)
	{
		XDBG(DEBUG_ASIC,"Asic_Close: Scanner is not opened");
		return STATUS_GOOD;
	}
	
	if (chip->firmwarestate > FS_OPENED)
	{
		XDBG(DEBUG_ASIC,"Asic_Close: Scanner is scanning, try to stop scanning");
		Asic_ScanStop(chip);
	}
	
	Mustek_SendData(chip,ES01_86_DisableAllClockWhenIdle, CLOSE_ALL_CLOCK_ENABLE);
	
	status = CloseScanChip(chip);
	if (status != STATUS_GOOD)
	{
		fprintf(stderr, "Asic_Close: CloseScanChip error\n");
		return status;
	}
	
	libusb_close(chip->fd);
	chip->firmwarestate = FS_ATTACHED;
	
	XDBG(DEBUG_ASIC,"Asic_Close: Exit");
	return status;
}

STATUS Asic_TurnLamp(PAsic chip, BOOL isLampOn)
{
	STATUS status = STATUS_GOOD;
	XDBG(DEBUG_ASIC,"Asic_TurnLamp: Enter");


	if (chip->firmwarestate < FS_OPENED)
	{
		fprintf(stderr,"Asic_TurnLamp: Scanner is not opened\n");
		return STATUS_INVAL;
	}

	if (chip->firmwarestate > FS_OPENED)
	{
		Mustek_SendData(chip,ES01_F4_ActiveTriger,ACTION_TRIGER_DISABLE);
	}

	BYTE PWM;
	if( isLampOn )
	{
		PWM=LAMP0_PWM_DEFAULT;
	}
	else
	{
		PWM=0;
	}
	Mustek_SendData(chip,ES01_99_LAMP_PWM_FREQ_CONTROL, 1);
	Mustek_SendData(chip,ES01_90_Lamp0PWM, PWM);
	XDBG(DEBUG_ASIC,"Lamp0 PWM = %d", PWM);
	
	chip->firmwarestate = FS_OPENED;
	
	XDBG(DEBUG_ASIC,"Asic_TurnLamp: Exit");
	return status;
}


STATUS Asic_TurnTA(PAsic chip, BOOL isTAOn)
{
	STATUS status = STATUS_GOOD;
	XDBG(DEBUG_ASIC,"Asic_TurnTA: Enter");

	if (chip->firmwarestate < FS_OPENED)
	{
		fprintf(stderr,"Asic_TurnTA: Scanner is not opened\n");
		return STATUS_INVAL;
	}

	if (chip->firmwarestate > FS_OPENED)
		Mustek_SendData(chip,ES01_F4_ActiveTriger,ACTION_TRIGER_DISABLE);

	BYTE PWM;
	if( isTAOn )
	{
		PWM=LAMP1_PWM_DEFAULT;		
	}
	else
	{
		PWM=0;

	}
	Mustek_SendData(chip,ES01_99_LAMP_PWM_FREQ_CONTROL, 1);
	Mustek_SendData(chip,ES01_91_Lamp1PWM, PWM);
	XDBG(DEBUG_ASIC,"Lamp1 PWM = %d", PWM);

	chip->firmwarestate = FS_OPENED;
	XDBG(DEBUG_ASIC,"Asic_TurnTA: Exit");
	return STATUS_GOOD;
}

STATUS Asic_WaitUnitReady(PAsic chip)
{
	STATUS status = STATUS_GOOD;
	XDBG(DEBUG_ASIC,"Asic_WaitUnitReady:Enter");
	BYTE temp_status;
	
	if(chip->firmwarestate < FS_OPENED)
	{
		fprintf(stderr,"Asic_WaitUnitReady: Scanner has not been opened\n");

		return STATUS_INVAL;
	}

	
	int i=0;
	do
	{
		status =GetChipStatus(chip,1, &temp_status);
		if (status != STATUS_GOOD)
		{
			XDBG(DEBUG_ASIC,"WaitChipIdle:Error!");
			return status;
		}
		i++;
		usleep(100000);
	}while( ( (temp_status & 0x1f) != 0) && i<300);
	XDBG(DEBUG_ASIC,"Wait %d s",(WORD)(i*0.1));
	
	Mustek_SendData(chip,ES01_F4_ActiveTriger, ACTION_TRIGER_DISABLE);
	chip->motorstate=MS_STILL;

	XDBG(DEBUG_ASIC,"Asic_WaitUnitReady: Exit");
	return status;
}

STATUS Asic_Release(PAsic chip)
{
	STATUS status = STATUS_GOOD;
	XDBG(DEBUG_ASIC,"Asic_Release()");
	
	if (chip->firmwarestate > FS_ATTACHED)
		status = Asic_Close(chip);
	
	chip->firmwarestate=FS_NULL;
	
	XDBG(DEBUG_ASIC,"Asic_Release: Exit");
	return status;
}


STATUS Asic_Initialize(PAsic chip)
{
	STATUS status=STATUS_GOOD;
	XDBG(DEBUG_ASIC,"Asic_Initialize:Enter");

  	
	chip->motorstate = MS_STILL;
	chip->dwBytesCountPerRow = 0;
	chip->lpGammaTable=NULL;
  	XDBG(DEBUG_ASIC,"isFirstOpenChip=%d",chip->isFirstOpenChip);

	chip->isFirstOpenChip=TRUE;
  	XDBG(DEBUG_ASIC,"isFirstOpenChip=%d",chip->isFirstOpenChip);

	chip->SWWidth = 0;
	chip->TA_Status = TA_UNKNOW;
	chip->lpShadingTable=NULL;
	chip->isMotorMove = MOTOR_0_ENABLE;


	Asic_Reset(chip);
	InitTiming(chip);

	chip->isUniformSpeedToScan=UNIFORM_MOTOR_AND_SCAN_SPEED_DISABLE;
	chip->isMotorGoToFirstLine=MOTOR_MOVE_TO_FIRST_LINE_ENABLE;//


	chip->UsbHost = HT_USB10;

	XDBG(DEBUG_ASIC,"Asic_Initialize: Exit");
	return status;
}

STATUS Asic_SetWindow(PAsic chip, BYTE bScanBits, 
					  WORD wXResolution, WORD wYResolution, 
					  WORD wX, WORD wY, WORD wWidth, WORD wLength)
{
	STATUS status=STATUS_GOOD;
	XDBG(DEBUG_ASIC,"Asic_SetWindow: Enter");
	XDBG(DEBUG_ASIC,"bScanBits=%d,wXResolution=%d,wYResolution=%d,wX=%d,wY=%d,wWidth=%d,wLength=%d",
		bScanBits,wXResolution,wYResolution,wX,wY,wWidth,wLength);

	if (chip->firmwarestate != FS_OPENED)
	{
		fprintf(stderr,"Asic_SetWindow: Scanner is not opened\n");
		return STATUS_INVAL;
	}

    
	WORD ValidPixelNumber;
	BYTE  temp_status;

	WORD wPerLineNeedBufferSize = 0;
	WORD BytePerPixel = 0;
	DWORD dwTotal_PerLineNeedBufferSize = 0 ; 
	DWORD dwTotalLineTheBufferNeed = 0;
	WORD  dwTotal_CCDResolution = 1200;
	WORD  wThinkCCDResolution = 0;
	WORD  wCCD_PixelNumber = 0;
    WORD  wScanAccSteps = 1;
	BYTE  byScanDecSteps = 1;
	DWORD dwLineWidthPixel = 0;
	WORD  wNowMotorDPI;
	BYTE  DoAutoGainOffset = 0;
	WORD XRatioTypeWord;
	double XRatioTypeDouble;
	double XRatioAdderDouble;	
	int i = 0;

	LLF_MOTORMOVE *lpMotorStepsTable=(LLF_MOTORMOVE *)malloc(sizeof(LLF_MOTORMOVE));
	BYTE byDummyCycleNum=0;

	Mustek_SendData(chip,ES01_F3_ActionOption, 0);
	Mustek_SendData(chip,ES01_86_DisableAllClockWhenIdle, CLOSE_ALL_CLOCK_DISABLE);
	Mustek_SendData(chip,ES01_F4_ActiveTriger, ACTION_TRIGER_DISABLE);

	status=Asic_WaitUnitReady(chip);

	/* dummy clock mode */
	Mustek_SendData(chip,0x1CD, 0);

	/* LED Flash */
	Mustek_SendData(chip,ES01_94_PowerSaveControl, 0x27 | 64 | 128); 

	/* calculate byte per line */
	if (bScanBits > 24)
	{     
		wPerLineNeedBufferSize = wWidth * 6;
		BytePerPixel = 6;
		chip->dwBytesCountPerRow=(DWORD)(wWidth)*6;
	}
	else if (bScanBits == 24)
	{
		wPerLineNeedBufferSize = wWidth  * 3;
		BytePerPixel = 3;
		chip->dwBytesCountPerRow=(DWORD)(wWidth)*3;
	}
	else if ( (bScanBits > 8) && (bScanBits <= 16) )
	{
		wPerLineNeedBufferSize = wWidth * 2;
		BytePerPixel = 2;
		chip->dwBytesCountPerRow=(DWORD)(wWidth)*2;
	}
	else if ((bScanBits == 8))
	{
		wPerLineNeedBufferSize = wWidth;
		BytePerPixel = 1;
		chip->dwBytesCountPerRow=(DWORD)(wWidth);
	}
	else if ((bScanBits < 8))
	{
		wPerLineNeedBufferSize = wWidth >> 3;
		BytePerPixel = 1;
		chip->dwBytesCountPerRow=(DWORD)(wWidth);
	}
	XDBG(DEBUG_ASIC,"dwBytesCountPerRow = %d",chip->dwBytesCountPerRow);

	byDummyCycleNum=0;
	if(chip->lsLightSource==LS_REFLECTIVE)
	{
		if(	chip->UsbHost==HT_USB10 )
		{
			switch(wYResolution)
			{
			case 2400:
			case 1200:
				if(chip->dwBytesCountPerRow>22000)
					byDummyCycleNum=4;
				else if(chip->dwBytesCountPerRow>15000)
					byDummyCycleNum=3;
				else if(chip->dwBytesCountPerRow>10000)
					byDummyCycleNum=2;
				else if(chip->dwBytesCountPerRow>5000)
					byDummyCycleNum=1;
				break;
			case 600:
			case 300:
			case 150:
			case 100:
				if(chip->dwBytesCountPerRow>21000)
					byDummyCycleNum=7;
				else if(chip->dwBytesCountPerRow>18000)
					byDummyCycleNum=6;
				else if(chip->dwBytesCountPerRow>15000)
					byDummyCycleNum=5;
				else if(chip->dwBytesCountPerRow>12000)
					byDummyCycleNum=4;
				else if(chip->dwBytesCountPerRow>9000)
					byDummyCycleNum=3;
				else if(chip->dwBytesCountPerRow>6000)
					byDummyCycleNum=2;
				else if(chip->dwBytesCountPerRow>3000)
					byDummyCycleNum=1;
				break;
			case 75:
			case 50:
				byDummyCycleNum=1;
				break;
			default:
				byDummyCycleNum=0;
				break;
			}
		}
		else
		{
			switch(wYResolution)
			{
			case 2400:
			case 1200:
			case 75:
			case 50:
				byDummyCycleNum=1;
				break;
			default:
				byDummyCycleNum=0;
				break;
			}
		}	
	}

	dwTotal_PerLineNeedBufferSize = wPerLineNeedBufferSize;
	dwTotalLineTheBufferNeed = wLength;

	chip->Scan.Dpi = wXResolution; 
	CCDTiming(chip);

	dwTotal_CCDResolution = SENSOR_DPI;

	if(chip->lsLightSource==LS_REFLECTIVE)
	{
		if (wXResolution > (dwTotal_CCDResolution/2))
		{/* full ccd resolution //1200dpi*/
			wThinkCCDResolution = dwTotal_CCDResolution;
			Mustek_SendData(chip,ES01_98_GPIOControl8_15, 0x01);
			Mustek_SendData(chip,ES01_96_GPIOValue8_15, 0x01);		

			wCCD_PixelNumber = chip->Timing.wCCDPixelNumber_1200;  
		}
		else
		{/*600dpi*/
			wThinkCCDResolution = dwTotal_CCDResolution/2;
			Mustek_SendData(chip,ES01_98_GPIOControl8_15, 0x01);
			Mustek_SendData(chip,ES01_96_GPIOValue8_15, 0x00);
			wCCD_PixelNumber = chip->Timing.wCCDPixelNumber_600; 
		}
	}
	else
	{
		if (wXResolution > (dwTotal_CCDResolution/2))
		{
			wThinkCCDResolution = dwTotal_CCDResolution;
			Mustek_SendData(chip,ES01_98_GPIOControl8_15, 0x01);
			Mustek_SendData(chip,ES01_96_GPIOValue8_15, 0x01);		
			wCCD_PixelNumber = TA_IMAGE_PIXELNUMBER;
		}   
		else
		{
			wThinkCCDResolution = dwTotal_CCDResolution/2;
			Mustek_SendData(chip,ES01_98_GPIOControl8_15, 0x01);
			Mustek_SendData(chip,ES01_96_GPIOValue8_15, 0x00);
			wCCD_PixelNumber = TA_IMAGE_PIXELNUMBER;
		}   
	}

	dwLineWidthPixel = wWidth;

	SetLineTimeAndExposure(chip);

	Mustek_SendData(chip,ES01_CB_CCDDummyCycleNumber, byDummyCycleNum);


	SetLEDTime(chip);
	
	/* calculate Y ratio */
    WORD Total_MotorDPI;
	Total_MotorDPI = 1200;
	wNowMotorDPI = Total_MotorDPI;


	/* SetADConverter */
	Mustek_SendData(chip,ES01_74_HARDWARE_SETTING, MOTOR1_SERIAL_INTERFACE_G10_8_ENABLE |
										  LED_OUT_G11_DISABLE | 
										  SLAVE_SERIAL_INTERFACE_G15_14_DISABLE |
										  SHUTTLE_CCD_DISABLE );

    /* set AFE*/
	Mustek_SendData(chip,ES01_9A_AFEControl, AD9826_AFE | AUTO_CHANGE_AFE_GAIN_OFFSET_DISABLE );
	SetAFEGainOffset(chip);

	Mustek_SendData(chip,ES01_F7_DigitalControl,	DIGITAL_REDUCE_DISABLE);

	/* calculate X ratio */
	XRatioTypeDouble = wXResolution; 
	XRatioTypeDouble /= wThinkCCDResolution; 
	XRatioAdderDouble = 1 / XRatioTypeDouble; 
	XRatioTypeWord = (WORD) (XRatioTypeDouble*32768); /* 32768 = 2^15 */

	XRatioAdderDouble = (double)(XRatioTypeWord) / 32768; 
	XRatioAdderDouble = 1/XRatioAdderDouble; 

	/* 0x8000 = 1.00000 -> get all pixel */
	Mustek_SendData(chip,ES01_9E_HorizontalRatio1to15LSB,	LOBYTE(XRatioTypeWord) );
	Mustek_SendData(chip,ES01_9F_HorizontalRatio1to15MSB,	HIBYTE(XRatioTypeWord) );

	/* SetMotorType */
	Mustek_SendData(chip,ES01_A6_MotorOption, MOTOR_0_ENABLE |
										MOTOR_1_DISABLE |
										HOME_SENSOR_0_ENABLE |
										ES03_TABLE_DEFINE);

	Mustek_SendData(chip,ES01_F6_MorotControl1, SPEED_UNIT_1_PIXEL_TIME |
										MOTOR_SYNC_UNIT_1_PIXEL_TIME);

	/* SetScanStepTable */
	WORD wMultiMotorStep = 1;
	BYTE bMotorMoveType = _MOTOR_MOVE_TYPE;

	/*set Motor move type*/
	  if(wYResolution>=1200)   
		bMotorMoveType = _8_TABLE_SPACE_FOR_1_DIV_2_STEP;

	switch(bMotorMoveType)
	{
	case _4_TABLE_SPACE_FOR_FULL_STEP:
		wMultiMotorStep = 1;
		break;

	case _8_TABLE_SPACE_FOR_1_DIV_2_STEP:
		wMultiMotorStep = 2;
		break;

	case _16_TABLE_SPACE_FOR_1_DIV_4_STEP:
		wMultiMotorStep = 4;
		break;

	case _32_TABLE_SPACE_FOR_1_DIV_8_STEP:
		wMultiMotorStep = 8;
		break;
	}
	wY *= wMultiMotorStep; 	

	SetScanMode(chip,bScanBits);



	/*set white shading int and dec*/  
	if(chip->lsLightSource==LS_REFLECTIVE)
		Mustek_SendData(chip,ES01_F8_WHITE_SHADING_DATA_FORMAT, ES01_SHADING_3_INT_13_DEC);
	else
		Mustek_SendData(chip,ES01_F8_WHITE_SHADING_DATA_FORMAT, ES01_SHADING_4_INT_12_DEC);
	
	BYTE byClear_Pulse_Width = 0;
	SetPackAddress(chip, wXResolution,wWidth,wX,XRatioAdderDouble,
			XRatioTypeDouble,byClear_Pulse_Width,&ValidPixelNumber);
	SetExtraSetting(chip,wXResolution,wCCD_PixelNumber,FALSE);

	/* calculate line time  */
	DWORD dwLinePixelReport;	
	BYTE	byPHTG_PulseWidth, byPHTG_WaitWidth;
	byPHTG_PulseWidth = chip->Timing.PHTG_PluseWidth;
	byPHTG_WaitWidth = chip->Timing.PHTG_WaitWidth;
	dwLinePixelReport = ((byClear_Pulse_Width + 1) * 2   +
					  (byPHTG_PulseWidth + 1)   * (1) +
					  (byPHTG_WaitWidth + 1)    * (1) +
					  (wCCD_PixelNumber + 1) ) * (byDummyCycleNum + 1);

	XDBG(DEBUG_ASIC,"Motor Time = %d", ( dwLinePixelReport * wYResolution / wNowMotorDPI) / wMultiMotorStep);
	if (( dwLinePixelReport * wYResolution / wNowMotorDPI) / wMultiMotorStep > 64000)
	{
		XDBG(DEBUG_ASIC, "Motor Time Over Flow !!!");
	}
	

	WORD StartSpeed, EndSpeed;
	LLF_CALCULATEMOTORTABLE CalMotorTable;
	LLF_MOTOR_CURRENT_AND_PHASE CurrentPhase;
	LLF_RAMACCESS RamAccess;
	DWORD dwStartAddr, dwEndAddr, dwTableBaseAddr, dwShadingTableAddr;
	EndSpeed = (WORD)(( dwLinePixelReport * wYResolution / wNowMotorDPI ) / wMultiMotorStep); 
	SetMotorStepTable(chip,lpMotorStepsTable,wY,dwTotalLineTheBufferNeed*wNowMotorDPI/wYResolution*wMultiMotorStep,wYResolution); //modified by Chester 92/04/08

	BYTE isMotorMoveToFirstLine=chip->isMotorGoToFirstLine;
	BYTE isUniformSpeedToScan=chip->isUniformSpeedToScan; 
	BYTE isScanBackTracking=SCAN_BACK_TRACKING_ENABLE;    


	if(EndSpeed>=20000)
	{
		Asic_MotorMove(chip,1,wY/wMultiMotorStep);
		isUniformSpeedToScan=UNIFORM_MOTOR_AND_SCAN_SPEED_ENABLE;
		isScanBackTracking=SCAN_BACK_TRACKING_DISABLE;
		isMotorMoveToFirstLine=MOTOR_MOVE_TO_FIRST_LINE_DISABLE;
	}

	Mustek_SendData(chip,ES01_F3_ActionOption,	isMotorMoveToFirstLine |
											MOTOR_BACK_HOME_AFTER_SCAN_DISABLE |
											SCAN_ENABLE |
											isScanBackTracking| 
											INVERT_MOTOR_DIRECTION_DISABLE |
											isUniformSpeedToScan |
											ES01_STATIC_SCAN_DISABLE |
											MOTOR_TEST_LOOP_DISABLE );

	if (EndSpeed > 8000)
	{
		StartSpeed = EndSpeed;
	}
	else
	{
		if(EndSpeed <= 1000)
			StartSpeed = EndSpeed + 4500;
		else
			StartSpeed = EndSpeed + 3500;
	}

	Mustek_SendData(chip,ES01_FD_MotorFixedspeedLSB, LOBYTE(EndSpeed));
	Mustek_SendData(chip,ES01_FE_MotorFixedspeedMSB, HIBYTE(EndSpeed));
	
	LPWORD lpMotorTable = (unsigned short *) malloc (512 * 8 * 2);
	memset(lpMotorTable, 0, 512 * 8 * sizeof(WORD));
	
	CalMotorTable.StartSpeed = StartSpeed;
	CalMotorTable.EndSpeed = EndSpeed;
	CalMotorTable.AccStepBeforeScan = lpMotorStepsTable->wScanAccSteps;
	CalMotorTable.DecStepAfterScan = lpMotorStepsTable->bScanDecSteps;
	CalMotorTable.lpMotorTable = lpMotorTable;

	CalculateMotorTable(&CalMotorTable,wYResolution);

	CurrentPhase.MotorDriverIs3967 = 0;
	CurrentPhase.FillPhase = 1;
	CurrentPhase.MoveType = bMotorMoveType;
	SetMotorCurrent(chip,EndSpeed,&CurrentPhase);
	LLFSetMotorCurrentAndPhase (chip,&CurrentPhase);

	XDBG(DEBUG_ASIC,"EndSpeed = %d, BytesCountPerRow=%d, MotorCurrentTable=%d, LinePixelReport=%d", 

		    EndSpeed,chip->dwBytesCountPerRow,CurrentPhase.MotorCurrentTableA[0],dwLinePixelReport);

	DWORD RealTableSize;

	/* write motor table */
	RealTableSize = 512*8;
	dwTableBaseAddr = PackAreaStartAddress - RealTableSize;

	dwStartAddr	= dwTableBaseAddr;

	RamAccess.ReadWrite = WRITE_RAM;
	RamAccess.IsOnChipGamma = EXTERNAL_RAM;
	RamAccess.DramDelayTime = SDRAMCLK_DELAY_12_ns;
	RamAccess.LoStartAddress = (WORD) (dwStartAddr);
	RamAccess.HiStartAddress = (WORD) (dwStartAddr>>16);
	RamAccess.RwSize = RealTableSize*2;
	RamAccess.BufferPtr = (LPBYTE)lpMotorTable;
	LLFRamAccess(chip,&RamAccess);
	
	Mustek_SendData(chip,ES01_DB_PH_RESET_EDGE_TIMING_ADJUST, 0x00);
	
	Mustek_SendData(chip,ES01_DC_CLEAR_EDGE_TO_PH_TG_EDGE_WIDTH, 0);

	Mustek_SendData(chip,ES01_9D_MotorTableAddrA14_A21, (BYTE)(dwTableBaseAddr>>TABLE_OFFSET_BASE) );		


	/* set address and shading table */
	/*set image buffer range and write shading table */
	RealTableSize = (ACC_DEC_STEP_TABLE_SIZE*NUM_OF_ACC_DEC_STEP_TABLE) + ShadingTableSize(ValidPixelNumber);
	dwShadingTableAddr = PackAreaStartAddress - ((( RealTableSize + (TABLE_BASE_SIZE - 1) )>>TABLE_OFFSET_BASE)<<TABLE_OFFSET_BASE);
	dwEndAddr = dwShadingTableAddr - 1;
	dwStartAddr	= dwShadingTableAddr;

	double dbXRatioAdderDouble;
	if(wXResolution > 600)
		dbXRatioAdderDouble = 1200 / wXResolution;
	else
		dbXRatioAdderDouble = 600 / wXResolution;
	
	RamAccess.ReadWrite = WRITE_RAM;
	RamAccess.IsOnChipGamma = EXTERNAL_RAM;
	RamAccess.DramDelayTime = SDRAMCLK_DELAY_12_ns;
	RamAccess.LoStartAddress = (WORD) (dwStartAddr);
	RamAccess.HiStartAddress = (WORD) (dwStartAddr>>16);
	RamAccess.RwSize = ShadingTableSize((int)((wWidth+4)*dbXRatioAdderDouble))*sizeof(WORD);
	RamAccess.BufferPtr = (BYTE*)chip->lpShadingTable;
	LLFRamAccess(chip,&RamAccess);

	/*tell scan chip the shading table address, unit is 2^15 bytes(2^14 word)*/
	Mustek_SendData(chip,ES01_9B_ShadingTableAddrA14_A21, (BYTE)(dwShadingTableAddr>>TABLE_OFFSET_BASE) );

	WORD wFullBank;
	/*empty bank*/
	Mustek_SendData(chip,ES01_FB_BufferEmptySize16WordLSB, LOBYTE(WaitBufferOneLineSize>>(7-3)));
	Mustek_SendData(chip,ES01_FC_BufferEmptySize16WordMSB, HIBYTE(WaitBufferOneLineSize>>(7-3)));

	/*full bank*/
	wFullBank = (WORD)((dwEndAddr - ( ((dwLineWidthPixel*BytePerPixel)/2)*3*1 )) / BANK_SIZE); 
	Mustek_SendData(chip,ES01_F9_BufferFullSize16WordLSB, LOBYTE(wFullBank));
	Mustek_SendData(chip,ES01_FA_BufferFullSize16WordMSB, HIBYTE(wFullBank));


	/* set buffer address */
	LLFSetRamAddress(chip,0x0, dwEndAddr, ACCESS_DRAM); 

    Mustek_SendData(chip,ES01_00_ADAFEConfiguration,	0x70);
    Mustek_SendData(chip,ES01_02_ADAFEMuxConfig,		0x80);
	
	free(lpMotorTable);	
	free(lpMotorStepsTable);
	chip->firmwarestate=FS_OPENED;

	XDBG(DEBUG_ASIC,"Asic_SetWindow: Exit");
	return status;
}

STATUS Asic_Reset(PAsic chip)
{
	STATUS status=STATUS_GOOD;
	XDBG(DEBUG_ASIC,"Asic_Reset: Enter");

	chip->lsLightSource=LS_REFLECTIVE;

	chip->dwBytesCountPerRow=0;
	chip->AD.DirectionR=0;
	chip->AD.DirectionG=0;
	chip->AD.DirectionB=0;
	chip->AD.GainR=0;
	chip->AD.GainG=0;
	chip->AD.GainB=0;

	chip->AD.OffsetR=0;
	chip->AD.OffsetG=0;
	chip->AD.OffsetB=0;

	chip->Scan.TotalMotorSteps=60000;
	chip->Scan.StartLine=0;
	chip->Scan.StartPixel=0;

	XDBG(DEBUG_ASIC,"Asic_Reset: Exit");
	return status;
}

STATUS Asic_SetSource(PAsic chip, LIGHTSOURCE lsLightSource)
{
	STATUS status=STATUS_GOOD;
	XDBG(DEBUG_ASIC,"Asic_SetSource: Enter");

	chip->lsLightSource = lsLightSource;
	switch(chip->lsLightSource)
	{
	case 1:
		XDBG(DEBUG_ASIC, "Asic_SetSource: Source is Reflect");
		break;
	case 2:
		XDBG(DEBUG_ASIC, "Asic_SetSource: Source is Postion");
		break;
	case 4:
		XDBG(DEBUG_ASIC, "Asic_SetSource: Source is Negtive");
		break;
	default:
		XDBG(DEBUG_ASIC,"Asic_SetSource: Source error");
	}

	XDBG(DEBUG_ASIC,"Asic_SetSource: Exit");
	return status;
}

STATUS Asic_ScanStart(PAsic chip)
{
	STATUS status=STATUS_GOOD;
	XDBG(DEBUG_ASIC,"Asic_ScanStart: Enter");

	if (chip->firmwarestate != FS_OPENED)
	{
		fprintf(stderr,"Asic_ScanStart: Scanner is not opened\n");
		return STATUS_INVAL;
	}

	Mustek_SendData(chip,ES01_8B_Status, 0x1c | 0x20);
	Mustek_WriteAddressLineForRegister(chip,0x8B);
	Mustek_ClearFIFO(chip);
	Mustek_SendData(chip,ES01_F4_ActiveTriger, ACTION_TRIGER_ENABLE);


	chip->firmwarestate = FS_SCANNING;

	XDBG(DEBUG_ASIC,"Asic_ScanStart: Exit");
	return status;
}

STATUS Asic_ScanStop(PAsic chip)
{
	STATUS status = STATUS_GOOD;
	XDBG(DEBUG_ASIC,"Asic_ScanStop: Enter");

	BYTE temps[2];
	BYTE buf[4];  

	if (chip->firmwarestate < FS_SCANNING)
	{

		return status;
	}
	
	buf[0] = 0x02;/*stop*/
	buf[1] = 0x02;
	buf[2] = 0x02;
	buf[3] = 0x02;
	status = WriteIOControl(chip,0xc0,0,4,buf);
	if (status != STATUS_GOOD)
	{
		fprintf(stderr,"Asic_ScanStop: Stop scan error\n");
		return status;
	}

	buf[0] = 0x00;/*clear*/
	buf[1] = 0x00;
	buf[2] = 0x00;
	buf[3] = 0x00;
	status = WriteIOControl(chip,0xc0,0,4,buf);
	if (status != STATUS_GOOD)
	{
		fprintf(stderr,"Asic_ScanStop: Clear scan error\n");
		return status;
	}

	status = Mustek_DMARead(chip,2, temps);
	if (status != STATUS_GOOD)
	{
		fprintf(stderr,"Asic_ScanStop: DMAReadGeneralMode error\n");
		return status;
	}

	Mustek_SendData(chip,ES01_F3_ActionOption, 0);
	Mustek_SendData(chip,ES01_86_DisableAllClockWhenIdle, CLOSE_ALL_CLOCK_DISABLE);
	Mustek_SendData(chip,ES01_F4_ActiveTriger, ACTION_TRIGER_DISABLE);
	Mustek_ClearFIFO(chip);
	
	chip->firmwarestate = FS_OPENED;
	XDBG(DEBUG_ASIC,"Asic_ScanStop: Exit");
	return status;
}

STATUS Asic_ReadImage(PAsic chip, LPBYTE pBuffer, WORD LinesCount)
{
	STATUS status = STATUS_GOOD;
	XDBG(DEBUG_ASIC,"Asic_ReadImage: Enter : LinesCount = %d",LinesCount);
	DWORD dwXferBytes;

	if (chip->firmwarestate != FS_SCANNING)
	{
		fprintf(stderr,"Asic_ReadImage: Scanner is not scanning\n");
		return STATUS_INVAL;
	}

	dwXferBytes = (DWORD)(LinesCount) *chip->dwBytesCountPerRow;
	XDBG(DEBUG_ASIC,"Asic_ReadImage: chip->dwBytesCountPerRow = %d",chip->dwBytesCountPerRow);
	
	if (dwXferBytes <0)
	{
		XDBG(DEBUG_ASIC,"Asic_ReadImage: dwXferBytes <0");
		return STATUS_INVAL;
	}
	
	if (dwXferBytes == 0)

	{
		XDBG(DEBUG_ASIC,"Asic_ReadImage: dwXferBytes == 0");
		return STATUS_GOOD;
	}

	status = Mustek_DMARead(chip,dwXferBytes, pBuffer);

	XDBG(DEBUG_ASIC,"Asic_ReadImage: Exit");
	return status;
}


STATUS Asic_CheckFunctionKey(PAsic chip,LPBYTE key)
{
	STATUS status = STATUS_GOOD;
	XDBG(DEBUG_ASIC,"Asic_CheckFunctionKey: Enter");
	BYTE bBuffer_1=0xff;
	BYTE bBuffer_2=0xff;


	if (chip->firmwarestate != FS_OPENED)
	{
		fprintf(stderr,"Asic_CheckFunctionKey: Scanner is not Opened\n");
		return STATUS_INVAL;
	}
	
	Mustek_SendData(chip,ES01_97_GPIOControl0_7, 0x00);
	Mustek_SendData(chip,ES01_95_GPIOValue0_7, 0x17);
	Mustek_SendData(chip,ES01_98_GPIOControl8_15, 0x00);
	Mustek_SendData(chip,ES01_96_GPIOValue8_15, 0x08);
	
	GetChipStatus(chip,0x02,&bBuffer_1);
	GetChipStatus(chip,0x03,&bBuffer_2);
		
	
	if(((0xff-bBuffer_1)&0x10)==0x10)
		*key = 0x01;

	if(((0xff-bBuffer_1)&0x01)==0x01)
		*key = 0x02;
	if(((0xff-bBuffer_1)&0x04)==0x04)
		*key = 0x04;
	if(((0xff-bBuffer_2)&0x08)==0x08)
		*key = 0x08;
	if(((0xff-bBuffer_1)&0x02)==0x02)
		*key = 0x10;

	
	XDBG(DEBUG_ASIC,"CheckFunctionKey=%d",*key);
	XDBG(DEBUG_ASIC,"Asic_CheckFunctionKey: Exit");
	return status;
}

STATUS Asic_IsTAConnected(PAsic chip,PBOOL hasTA)
{
	STATUS status = STATUS_GOOD;
	XDBG(DEBUG_ASIC,"Asic_IsTAConnected: Enter");

	BYTE bBuffer_1=0xff;
	BYTE bBuffer_2=0xff;
	
	Mustek_SendData(chip,ES01_97_GPIOControl0_7, 0x00);
	Mustek_SendData(chip,ES01_95_GPIOValue0_7, 0x00);

	Mustek_SendData(chip,ES01_98_GPIOControl8_15, 0x00);
	Mustek_SendData(chip,ES01_96_GPIOValue8_15, 0x00);
	

	GetChipStatus(chip,0x02,&bBuffer_1);  
	
	if(((0xff-bBuffer_1)&0x08)==0x08)
		*hasTA = TRUE;
	else
		*hasTA = FALSE;

	XDBG(DEBUG_ASIC,"hasTA=%d",*hasTA);
	XDBG(DEBUG_ASIC,"Asic_IsTAConnected():Exit");
	return STATUS_GOOD;
}

STATUS Asic_DownloadGammaTable(PAsic chip, LPVOID lpBuffer)
{
	STATUS status = STATUS_GOOD;
	XDBG(DEBUG_ASIC,"Asic_DownloadGammaTable()");
	
	chip->lpGammaTable = lpBuffer;

	XDBG(DEBUG_ASIC,"Asic_DownloadGammaTable: Exit");
	return status;
}

STATUS Asic_ReadCalibrationData(PAsic chip, LPVOID pBuffer, 
								DWORD dwXferBytes,BYTE bScanBits)
{
	STATUS status = STATUS_GOOD;
	XDBG(DEBUG_ASIC,"Asic_ReadCalibrationData: Enter");

	LPBYTE	pCalBuffer;
	DWORD	dwTotalReadData;
	DWORD	dwReadImageData;

	if (chip->firmwarestate != FS_SCANNING)
	{
		fprintf(stderr,"Asic_ReadCalibrationData: Scanner is not scanning\n");
		return STATUS_INVAL;
	}
	
	if(bScanBits==24)
	{
		DWORD i;
		pCalBuffer = (LPBYTE)malloc(dwXferBytes);
		if(pCalBuffer==NULL)
		{
			fprintf(stderr,"Asic_ReadCalibrationData: Can't malloc bCalBuffer memory\n");
			return STATUS_MEM_ERROR;
		}

		for(dwTotalReadData=0; dwTotalReadData<dwXferBytes; )
		{
			dwReadImageData=(dwXferBytes-dwTotalReadData)< 65536 ? 
				            (dwXferBytes-dwTotalReadData): 65536 ;
			
			Mustek_DMARead(chip,dwReadImageData, (LPBYTE)(pCalBuffer+dwTotalReadData));
			dwTotalReadData+=dwReadImageData;
		}

		dwXferBytes/=3;
		for(i=0; i<dwXferBytes; i++)

		{
			*((LPBYTE)pBuffer+i)  =*(pCalBuffer+i*3);
			*((LPBYTE)pBuffer+dwXferBytes+i)=*(pCalBuffer+i*3+1);
			*((LPBYTE)pBuffer+dwXferBytes*2+i)=*(pCalBuffer+i*3+2);
		}
		free(pCalBuffer);
	}
	else if(bScanBits==8)
	{
		for(dwTotalReadData=0; dwTotalReadData<dwXferBytes; )
		{
			dwReadImageData=(dwXferBytes-dwTotalReadData)< 65536 ? 
				            (dwXferBytes-dwTotalReadData): 65536 ;
			
			Mustek_DMARead(chip,dwReadImageData, (LPBYTE)pBuffer+dwTotalReadData);
			dwTotalReadData+=dwReadImageData;
		}
	}

	XDBG(DEBUG_ASIC,"Asic_ReadCalibrationData: Exit");
	return status;
}

STATUS Asic_SetMotorType(PAsic chip, 
						BOOL isMotorMove,BOOL isUniformSpeed)
{
	STATUS status = STATUS_GOOD;
	XDBG(DEBUG_ASIC,"Asic_SetMotorType:Enter");

	if(isMotorMove)
	{
		chip->isMotorMove=MOTOR_0_ENABLE;
	}
	else
		chip->isMotorMove= MOTOR_0_DISABLE; 

	XDBG(DEBUG_ASIC,"isMotorMove=%d",chip->isMotorMove);
	XDBG(DEBUG_ASIC,"Asic_SetMotorType: Exit");
	return status;
}

STATUS Asic_MotorMove(PAsic chip, BOOL isForward,DWORD dwTotalSteps)
{
	STATUS status = STATUS_GOOD;
	XDBG(DEBUG_ASIC,"Asic_MotorMove:Enter");

	WORD *NormalMoveMotorTable;
	
	NormalMoveMotorTable = (LPWORD)malloc(512*8*2);
          
	LLF_CALCULATEMOTORTABLE CalMotorTable;
	CalMotorTable.StartSpeed = 5000;
	CalMotorTable.EndSpeed = 1800;
	CalMotorTable.AccStepBeforeScan = 511;
	CalMotorTable.lpMotorTable = NormalMoveMotorTable;
	LLFCalculateMotorTable(&CalMotorTable);
	
	LLF_MOTOR_CURRENT_AND_PHASE CurrentPhase;
	CurrentPhase.MotorDriverIs3967=0;
	CurrentPhase.MotorCurrentTableA[0] = 200;
	CurrentPhase.MotorCurrentTableB[0] = 200;
	CurrentPhase.MoveType = _4_TABLE_SPACE_FOR_FULL_STEP;
	LLFSetMotorCurrentAndPhase (chip,&CurrentPhase);
	
	
	LLF_SETMOTORTABLE LLF_SetMotorTable;
	LLF_SetMotorTable.MotorTableAddress = 0;
	LLF_SetMotorTable.MotorTablePtr = NormalMoveMotorTable;
	LLFSetMotorTable (chip,&LLF_SetMotorTable);
	
	free(NormalMoveMotorTable);
	LLF_MOTORMOVE MotorMove;
	
	MotorMove.MotorSelect = MOTOR_0_ENABLE | MOTOR_1_DISABLE;
	MotorMove.MotorMoveUnit = ES03_TABLE_DEFINE;
	MotorMove.MotorSpeedUnit = SPEED_UNIT_1_PIXEL_TIME;
	MotorMove.MotorSyncUnit = MOTOR_SYNC_UNIT_1_PIXEL_TIME;
	MotorMove.HomeSensorSelect = HOME_SENSOR_0_ENABLE;
	MotorMove.ActionMode = ACTION_MODE_ACCDEC_MOVE;
	MotorMove.ActionType = isForward;
	
	if (dwTotalSteps > 1000)
	{
		MotorMove.AccStep = 511;
		MotorMove.DecStep = 255;
		MotorMove.FixMoveSteps = dwTotalSteps - (511+255);
	}
	else
	{

		MotorMove.ActionMode = ACTION_MODE_UNIFORM_SPEED_MOVE;
		MotorMove.AccStep = 1;
		MotorMove.DecStep = 1;
		MotorMove.FixMoveSteps = dwTotalSteps - 2;
	}
	
	MotorMove.FixMoveSpeed = 7000;
	MotorMove.WaitOrNoWait = TRUE;
	
	LLFMotorMove (chip,&MotorMove);
	
	XDBG(DEBUG_ASIC,"Asic_MotorMove: Exit");
	return status;
}

STATUS Asic_CarriageHome(PAsic chip, BOOL isTA)
{
	STATUS status = STATUS_GOOD;
	XDBG(DEBUG_ASIC,"Asic_CarriageHome:Enter");
	BOOL LampHome, TAHome;

	status =IsCarriageHome(chip,&LampHome,&TAHome);
	if (!LampHome)
	{
		status =MotorBackHome(chip,TRUE);
	}
	
	XDBG(DEBUG_ASIC,"Asic_CarriageHome: Exit");
	return status;
}

STATUS Asic_SetShadingTable(PAsic chip, LPWORD lpWhiteShading, 
								LPWORD lpDarkShading, 
								WORD wXResolution, WORD wWidth, 
								WORD wX)
{
	STATUS status = STATUS_GOOD;
	XDBG(DEBUG_ASIC,"Asic_SetShadingTable:Enter");

	WORD i,j,n;
	WORD wIndex=1;
	WORD wValidPixelNumber;
    double dbXRatioAdderDouble;
	DWORD   wShadingTableSize; 
	
	if(chip->firmwarestate<FS_OPENED)

		OpenScanChip(chip);
	if(chip->firmwarestate==FS_SCANNING)
		Mustek_SendData(chip,ES01_F4_ActiveTriger, ACTION_TRIGER_DISABLE);
	
	
	if(wXResolution > 600)
		dbXRatioAdderDouble = 1200 / wXResolution;
	else
		dbXRatioAdderDouble = 600 / wXResolution;
	
	wValidPixelNumber = (WORD) ( (wWidth+4)*dbXRatioAdderDouble );
	XDBG(DEBUG_ASIC,"wValidPixelNumber = %d",wValidPixelNumber);
	
	/* clear old Shading table, if it has.*/
	/*  first 4 element and lastest 5 of Shading table can't been used*/
	wShadingTableSize = (ShadingTableSize(wValidPixelNumber))*sizeof(WORD) ; 
	if(chip->lpShadingTable != NULL)
	{
		free(chip->lpShadingTable);

	}

	XDBG(DEBUG_ASIC,"Alloc a new shading table= %d Byte!",wShadingTableSize);
	chip->lpShadingTable = (LPBYTE)malloc(wShadingTableSize);
	if(chip->lpShadingTable == NULL)
	{
		XDBG(DEBUG_ASIC,"lpShadingTable == NULL");
		return STATUS_MEM_ERROR;
	}
	
	n = 0;
	for (i = 0 ; i <= (wValidPixelNumber / 40); i++)
	{
		if( i < (wValidPixelNumber / 40))
		{			
			for (j = 0; j < 40; j++)
			{
				*((LPWORD)chip->lpShadingTable + i * 256 + j * 6	  ) = *((LPWORD)lpDarkShading + n * 3);
				*((LPWORD)chip->lpShadingTable + i * 256 + j * 6 + 2) = *((LPWORD)lpDarkShading + n * 3 + 1);
				*((LPWORD)chip->lpShadingTable + i * 256 + j * 6 + 4) = *((LPWORD)lpDarkShading + n * 3 + 2);
				
				*((LPWORD)chip->lpShadingTable + i * 256 + j * 6 + 1) = *((LPWORD)lpWhiteShading + n * 3);
				*((LPWORD)chip->lpShadingTable + i * 256 + j * 6 + 3) = *((LPWORD)lpWhiteShading + n * 3 + 1);
				*((LPWORD)chip->lpShadingTable + i * 256 + j * 6 + 5) = *((LPWORD)lpWhiteShading + n * 3 + 2);
				if((j % (WORD)dbXRatioAdderDouble) == (dbXRatioAdderDouble-1)) n++;
				
				if(i==0 && j<4*dbXRatioAdderDouble)
					n=0;
			}
		}
		else
		{
			for (j = 0; j < (wValidPixelNumber % 40); j++)
			{
				*((LPWORD)chip->lpShadingTable + i * 256 + j * 6	  ) = *((LPWORD)lpDarkShading +(  n) * 3);
				*((LPWORD)chip->lpShadingTable + i * 256 + j * 6 + 2) = *((LPWORD)lpDarkShading +(  n) * 3 + 1);
				*((LPWORD)chip->lpShadingTable + i * 256 + j * 6 + 4) = *((LPWORD)lpDarkShading +(  n) * 3 + 2);
				
				*((LPWORD)chip->lpShadingTable + i * 256 + j * 6 + 1) = *((LPWORD)lpWhiteShading +(  n) * 3);
				*((LPWORD)chip->lpShadingTable + i * 256 + j * 6 + 3) = *((LPWORD)lpWhiteShading +(  n) * 3 + 1);
				*((LPWORD)chip->lpShadingTable + i * 256 + j * 6 + 5) = *((LPWORD)lpWhiteShading +(  n) * 3 + 2);
				
				if((j % (WORD)dbXRatioAdderDouble) == (dbXRatioAdderDouble-1)) n++;
				
				if(i==0 && j<4*dbXRatioAdderDouble)
					n=0;
			}
		}
	}

	XDBG(DEBUG_ASIC,"Asic_SetShadingTable: Exit");
	return status;
}

STATUS Asic_WaitCarriageHome(PAsic chip, BOOL isTA)
{
	STATUS status = STATUS_GOOD;
	XDBG(DEBUG_ASIC,"Asic_WaitCarriageHome:Enter");
	BOOL LampHome, TAHome;


	int i;
	for(i=0; i<100;i++)
	{
		status =IsCarriageHome(chip,&LampHome,&TAHome);
		if(LampHome)
			break;
		usleep(300000);
	}
	if(i==100)	status==STATUS_DEVICE_BUSY;
	XDBG(DEBUG_ASIC,"Wait %d s",(WORD)(i*0.3));

	Mustek_SendData(chip,ES01_F4_ActiveTriger, ACTION_TRIGER_DISABLE);
	chip->firmwarestate = FS_OPENED;
	chip->motorstate = MS_STILL;

	XDBG(DEBUG_ASIC,"Asic_WaitCarriageHome: Exit");
	return status;
}

STATUS Asic_SetCalibrate(PAsic chip,BYTE bScanBits, WORD wXResolution, 
						 WORD wYResolution, WORD wX, WORD wY, 
						 WORD wWidth, WORD wLength, BOOL isShading)
{
	STATUS status = STATUS_GOOD;
	XDBG(DEBUG_ASIC,"Asic_SetCalibrate: Enter");
	XDBG(DEBUG_ASIC,"bScanBits=%d,wXResolution=%d, wYResolution=%d,	wX=%d, wY=%d, wWidth=%d, wLength=%d",
		bScanBits,wXResolution, wYResolution, wX, wY, wWidth, wLength);
	
	if (chip->firmwarestate != FS_OPENED)
	{
		fprintf(stderr,"Asic_SetCalibrate: Scanner is not opened\n");
		return STATUS_INVAL;
	}
   
	WORD ValidPixelNumber;
	BYTE  temp_status;

	WORD wPerLineNeedBufferSize = 0;
	WORD BytePerPixel = 0;
	DWORD dwTotal_PerLineNeedBufferSize = 0 ; 
	DWORD dwTotalLineTheBufferNeed = 0;
	WORD  dwTotal_CCDResolution = 0;
	WORD  wThinkCCDResolution = 0;
	WORD  wCCD_PixelNumber = 0;
    WORD  wScanAccSteps = 1;
	BYTE  byScanDecSteps = 1;
	DWORD dwLineWidthPixel = 0;

	WORD  wNowMotorDPI;
	BYTE  DoAutoGainOffset = 0;
	WORD XRatioTypeWord;
	double XRatioTypeDouble;
	double XRatioAdderDouble;	
	int i = 0;
	
	LLF_MOTORMOVE *lpMotorStepsTable=(LLF_MOTORMOVE*)malloc(sizeof(LLF_MOTORMOVE)); 
	if (lpMotorStepsTable=NULL)
	{
		fprintf(stderr,"Asic_SetCalibrate: insufficiency memory!");
		return STATUS_INVAL;
	}
	XDBG(DEBUG_ASIC,"malloc LLF_MOTORMOVE =%d Byte",(sizeof(LLF_MOTORMOVE)));
	
	Mustek_SendData(chip,ES01_F3_ActionOption, 0);
	Mustek_SendData(chip,ES01_86_DisableAllClockWhenIdle, CLOSE_ALL_CLOCK_DISABLE);
	Mustek_SendData(chip,ES01_F4_ActiveTriger, ACTION_TRIGER_DISABLE);

	status=Asic_WaitUnitReady(chip);

	Mustek_SendData(chip,0x1CD, 0);

	Mustek_SendData(chip,ES01_94_PowerSaveControl, 0x27 | 64 | 128);

	if (bScanBits > 24){      
		wPerLineNeedBufferSize = wWidth * 6;
		BytePerPixel = 6;
		chip->dwBytesCountPerRow=(DWORD)(wWidth)*6;
	}else if (bScanBits == 24){
		wPerLineNeedBufferSize = wWidth  * 3;
		chip->dwCalibrationBytesCountPerRow = wWidth  * 3;
		BytePerPixel = 3;
		chip->dwBytesCountPerRow=(DWORD)(wWidth)*3;
	}else if ( (bScanBits > 8) && (bScanBits <= 16) ){
		wPerLineNeedBufferSize = wWidth * 2;
		BytePerPixel = 2;
		chip->dwBytesCountPerRow=(DWORD)(wWidth)*2;
	}else if ((bScanBits == 8)){
		wPerLineNeedBufferSize = wWidth;
		BytePerPixel = 1;
		chip->dwBytesCountPerRow=(DWORD)(wWidth);
	}else if ((bScanBits < 8)){
		wPerLineNeedBufferSize = wWidth >> 3;
		BytePerPixel = 1;
		chip->dwBytesCountPerRow=(DWORD)(wWidth);
	}
	XDBG(DEBUG_ASIC,"wPerLineNeedBufferSize=%d,BytePerPixel=%d,dwBytesCountPerRow=%d",
				wPerLineNeedBufferSize,BytePerPixel,chip->dwBytesCountPerRow);
	

	dwTotal_PerLineNeedBufferSize = wPerLineNeedBufferSize;  
	dwTotalLineTheBufferNeed = wLength; 
	XDBG(DEBUG_ASIC,"wPerLineNeedBufferSize=%d,wLength=%d",wPerLineNeedBufferSize,wLength);

	
	chip->Scan.Dpi = wXResolution;
	CCDTiming(chip);
	
	dwTotal_CCDResolution = SENSOR_DPI;
	if(chip->lsLightSource==LS_REFLECTIVE)
	{
		if (wXResolution > (dwTotal_CCDResolution/2))
		{
			wThinkCCDResolution = dwTotal_CCDResolution;
			Mustek_SendData(chip,ES01_98_GPIOControl8_15, 0x01);
			Mustek_SendData(chip,ES01_96_GPIOValue8_15, 0x01);		
			wCCD_PixelNumber = chip->Timing.wCCDPixelNumber_1200;
		}
		else
		{
			wThinkCCDResolution = dwTotal_CCDResolution/2;
			Mustek_SendData(chip,ES01_98_GPIOControl8_15, 0x01);
			Mustek_SendData(chip,ES01_96_GPIOValue8_15, 0x00);
			wCCD_PixelNumber = chip->Timing.wCCDPixelNumber_600;
		}
	}
	else
	{
		if (wXResolution > (dwTotal_CCDResolution/2)){
			wThinkCCDResolution = dwTotal_CCDResolution;
			Mustek_SendData(chip,ES01_98_GPIOControl8_15, 0x01);
			Mustek_SendData(chip,ES01_96_GPIOValue8_15, 0x01);		
			wCCD_PixelNumber = TA_CAL_PIXELNUMBER;
		}  
		else{         
			wThinkCCDResolution = dwTotal_CCDResolution/2;
			Mustek_SendData(chip,ES01_98_GPIOControl8_15, 0x01);
			Mustek_SendData(chip,ES01_96_GPIOValue8_15, 0x00);
			wCCD_PixelNumber =TA_CAL_PIXELNUMBER;
		}
	}
	XDBG(DEBUG_ASIC,"wThinkCCDResolution=%d,wCCD_PixelNumber=%d",wThinkCCDResolution,wCCD_PixelNumber);

	dwLineWidthPixel = wWidth;

	if(isShading)
		wYResolution=600;
	XDBG(DEBUG_ASIC,"dwLineWidthPixel=%d,wYResolution=%d",dwLineWidthPixel,wYResolution);
	
	BYTE byDummyCycleNum=1;
	SetLineTimeAndExposure(chip);
	if(wYResolution == 600)
	{
		Mustek_SendData(chip,ES01_CB_CCDDummyCycleNumber, byDummyCycleNum);
		XDBG(DEBUG_ASIC,"Find Boundary CCDDummyCycleNumber == %d",byDummyCycleNum);
	}

	SetLEDTime(chip);

	/* calculate Y ratio */
    WORD Total_MotorDPI;
	Total_MotorDPI = 1200;

	wNowMotorDPI = Total_MotorDPI;
	XDBG(DEBUG_ASIC,"wNowMotorDPI=%d",wNowMotorDPI);

	/* SetADConverter */
	Mustek_SendData(chip,ES01_74_HARDWARE_SETTING, MOTOR1_SERIAL_INTERFACE_G10_8_ENABLE |
										  LED_OUT_G11_DISABLE |
										  SLAVE_SERIAL_INTERFACE_G15_14_DISABLE |
										  SHUTTLE_CCD_DISABLE );

    /* set AFE*/
	Mustek_SendData(chip,ES01_9A_AFEControl, AD9826_AFE | AUTO_CHANGE_AFE_GAIN_OFFSET_DISABLE );
	Mustek_SendData(chip,ES01_F7_DigitalControl,	DIGITAL_REDUCE_DISABLE);

	XRatioTypeDouble = wXResolution;
	XRatioTypeDouble /= wThinkCCDResolution;
	XRatioAdderDouble = 1 / XRatioTypeDouble;
	XRatioTypeWord = (WORD) (XRatioTypeDouble*32768); 

	XRatioAdderDouble = (double)(XRatioTypeWord) / 32768; 
	XRatioAdderDouble = 1/XRatioAdderDouble; 

	Mustek_SendData(chip,ES01_9E_HorizontalRatio1to15LSB,	LOBYTE(XRatioTypeWord) );
	Mustek_SendData(chip,ES01_9F_HorizontalRatio1to15MSB,	HIBYTE(XRatioTypeWord) );
	XDBG(DEBUG_ASIC,"XRatioTypeDouble=%.2f,XRatioAdderDouble=%.2f,XRatioTypeWord=%d",
				XRatioTypeDouble,XRatioAdderDouble,XRatioTypeWord);
	
	if(chip->isMotorMove == MOTOR_0_ENABLE){
		Mustek_SendData(chip,ES01_A6_MotorOption, MOTOR_0_ENABLE |
			MOTOR_1_DISABLE |
			HOME_SENSOR_0_ENABLE |
			ES03_TABLE_DEFINE);
	}else{
		Mustek_SendData(chip,ES01_A6_MotorOption, MOTOR_0_DISABLE |
			MOTOR_1_DISABLE |
			HOME_SENSOR_0_ENABLE |
			ES03_TABLE_DEFINE);
	}
	XDBG(DEBUG_ASIC,"isMotorMove=%d",chip->isMotorMove);
	
	Mustek_SendData(chip,ES01_F6_MorotControl1, SPEED_UNIT_1_PIXEL_TIME |
										MOTOR_SYNC_UNIT_1_PIXEL_TIME);

	wScanAccSteps		= 1;
	byScanDecSteps		= 1;
	XDBG(DEBUG_ASIC,"wScanAccSteps=%d,byScanDecSteps=%d",wScanAccSteps,byScanDecSteps);
	
	Mustek_SendData(chip,ES01_AE_MotorSyncPixelNumberM16LSB,	LOBYTE(0)	);
	Mustek_SendData(chip,ES01_AF_MotorSyncPixelNumberM16MSB,	HIBYTE(0)	);
	XDBG(DEBUG_ASIC,"MotorSyncPixelNumber=%d",0);

	Mustek_SendData(chip,ES01_EC_ScanAccStep0_7, LOBYTE(wScanAccSteps) );
	Mustek_SendData(chip,ES01_ED_ScanAccStep8_8, HIBYTE(wScanAccSteps) );
	XDBG(DEBUG_ASIC,"wScanAccSteps=%d",wScanAccSteps);
	
    WORD BeforeScanFixSpeedStep = 0;
	WORD BackTrackFixSpeedStep  = 20;
	XDBG(DEBUG_ASIC,"BeforeScanFixSpeedStep=%d,BackTrackFixSpeedStep=%d",BeforeScanFixSpeedStep,BackTrackFixSpeedStep);

	Mustek_SendData(chip,ES01_EE_FixScanStepLSB, LOBYTE(BeforeScanFixSpeedStep) );
	Mustek_SendData(chip,ES01_8A_FixScanStepMSB, HIBYTE(BeforeScanFixSpeedStep) );
	XDBG(DEBUG_ASIC,"BeforeScanFixSpeedStep=%d",BeforeScanFixSpeedStep);

	Mustek_SendData(chip,ES01_EF_ScanDecStep, byScanDecSteps );
	XDBG(DEBUG_ASIC,"byScanDecSteps=%d",byScanDecSteps);

	Mustek_SendData(chip,ES01_E6_ScanBackTrackingStepLSB, LOBYTE(BackTrackFixSpeedStep) );
	Mustek_SendData(chip,ES01_E7_ScanBackTrackingStepMSB, HIBYTE(BackTrackFixSpeedStep) );
	XDBG(DEBUG_ASIC,"BackTrackFixSpeedStep=%d",BackTrackFixSpeedStep);

	Mustek_SendData(chip,ES01_E8_ScanRestartStepLSB, LOBYTE(BackTrackFixSpeedStep) );
	Mustek_SendData(chip,ES01_E9_ScanRestartStepMSB, HIBYTE(BackTrackFixSpeedStep) );
	XDBG(DEBUG_ASIC,"BackTrackFixSpeedStep=%d",BackTrackFixSpeedStep);

	WORD wMultiMotorStep = 1;
	BYTE bMotorMoveType = _MOTOR_MOVE_TYPE; 
	switch(bMotorMoveType)
	{
	case _4_TABLE_SPACE_FOR_FULL_STEP:
		wMultiMotorStep = 1;
		break;

		case _8_TABLE_SPACE_FOR_1_DIV_2_STEP:
		wMultiMotorStep = 2;
		break;

		case _16_TABLE_SPACE_FOR_1_DIV_4_STEP:
		wMultiMotorStep = 4;
		break;

		case _32_TABLE_SPACE_FOR_1_DIV_8_STEP:
		wMultiMotorStep = 8;
		break;
	}
	XDBG(DEBUG_ASIC,"wMultiMotorStep=%d",wMultiMotorStep);
	
	DWORD TotalStep = 0;
	TotalStep = wScanAccSteps + BeforeScanFixSpeedStep + 
				(dwTotalLineTheBufferNeed*wNowMotorDPI/wYResolution) + byScanDecSteps;  
	XDBG(DEBUG_ASIC,"TotalStep=%d",TotalStep);

	Mustek_SendData(chip,ES01_F0_ScanImageStep0_7,	(BYTE)(TotalStep));
	Mustek_SendData(chip,ES01_F1_ScanImageStep8_15,	(BYTE)(TotalStep>>8));
	Mustek_SendData(chip,ES01_F2_ScanImageStep16_19,	(BYTE)(TotalStep>>16));

	SetScanMode(chip,bScanBits);
	
	BYTE isMotorMoveToFirstLine=MOTOR_MOVE_TO_FIRST_LINE_DISABLE;   
	BYTE isUniformSpeedToScan=UNIFORM_MOTOR_AND_SCAN_SPEED_ENABLE; 
	BYTE isScanBackTracking=SCAN_BACK_TRACKING_DISABLE;            
	XDBG(DEBUG_ASIC,"isMotorMoveToFirstLine=%d,isUniformSpeedToScan=%d,isScanBackTracking=%d",
					isMotorMoveToFirstLine,isUniformSpeedToScan,isScanBackTracking);
	
    
	Mustek_SendData(chip,ES01_F3_ActionOption,	isMotorMoveToFirstLine |
											MOTOR_BACK_HOME_AFTER_SCAN_DISABLE |
											SCAN_ENABLE |
											isScanBackTracking| 
											INVERT_MOTOR_DIRECTION_DISABLE |
											isUniformSpeedToScan |
											ES01_STATIC_SCAN_DISABLE |
											MOTOR_TEST_LOOP_DISABLE );

	if(chip->lsLightSource==LS_REFLECTIVE)
		Mustek_SendData(chip,ES01_F8_WHITE_SHADING_DATA_FORMAT, ES01_SHADING_3_INT_13_DEC);
	else
		Mustek_SendData(chip,ES01_F8_WHITE_SHADING_DATA_FORMAT, ES01_SHADING_4_INT_12_DEC);

	BYTE byClear_Pulse_Width = 0;
	SetPackAddress(chip, wXResolution,wWidth,wX,XRatioAdderDouble,
			XRatioTypeDouble,byClear_Pulse_Width,&ValidPixelNumber);
	SetExtraSetting(chip,wXResolution,wCCD_PixelNumber,TRUE);
		
	DWORD dwLinePixelReport=0;	
	BYTE	byPHTG_PulseWidth, byPHTG_WaitWidth;
	byPHTG_PulseWidth = chip->Timing.PHTG_PluseWidth;
	byPHTG_WaitWidth = chip->Timing.PHTG_WaitWidth;
	dwLinePixelReport = ((byClear_Pulse_Width + 1) * 2   +
		(byPHTG_PulseWidth + 1)   * (1) +
		(byPHTG_WaitWidth + 1)    * (1) +
		(wCCD_PixelNumber + 1) ) * (byDummyCycleNum + 1);
	
	XDBG(DEBUG_ASIC,"Motor Time = %d", ( dwLinePixelReport * wYResolution / wNowMotorDPI));
	if ((dwLinePixelReport * wYResolution / wNowMotorDPI) > 64000){
    XDBG(DEBUG_ASIC, "Motor Time Over Flow !!!");
	}

	WORD StartSpeed, EndSpeed;
	LLF_CALCULATEMOTORTABLE CalMotorTable;
	LLF_MOTOR_CURRENT_AND_PHASE CurrentPhase;
	LLF_RAMACCESS RamAccess;
	DWORD dwStartAddr, dwEndAddr, dwTableBaseAddr;
	
	EndSpeed = (WORD)( dwLinePixelReport / (wNowMotorDPI / wYResolution ) );

	if (wXResolution > 600){
 		StartSpeed = EndSpeed;
	}else{
		StartSpeed = EndSpeed + 3500;
	}
	XDBG(DEBUG_ASIC,"StartSpeed =%d, EndSpeed = %d", StartSpeed, EndSpeed);


	Mustek_SendData(chip,ES01_FD_MotorFixedspeedLSB, LOBYTE(EndSpeed));
	Mustek_SendData(chip,ES01_FE_MotorFixedspeedMSB, HIBYTE(EndSpeed));
	LPWORD lpMotorTable = (unsigned short *) malloc (512 * 8 * 2);
	memset(lpMotorTable, 0, 512 * 8 * sizeof(WORD));
	
	CalMotorTable.StartSpeed = StartSpeed;
	CalMotorTable.EndSpeed = EndSpeed;
	CalMotorTable.AccStepBeforeScan = wScanAccSteps;
	CalMotorTable.lpMotorTable = lpMotorTable;

	LLFCalculateMotorTable(&CalMotorTable);

	CurrentPhase.MotorDriverIs3967 = 0;
	CurrentPhase.FillPhase = 1;
	CurrentPhase.MoveType = bMotorMoveType;
	CurrentPhase.MotorCurrentTableA[0] = 200;
	CurrentPhase.MotorCurrentTableB[0] = 200;
	LLFSetMotorCurrentAndPhase (chip,&CurrentPhase);

	DWORD RealTableSize;
	RealTableSize = 512*8;
	dwTableBaseAddr = PackAreaStartAddress - RealTableSize;

	dwStartAddr	= dwTableBaseAddr;

	RamAccess.ReadWrite = WRITE_RAM;
	RamAccess.IsOnChipGamma = EXTERNAL_RAM;
	RamAccess.DramDelayTime = SDRAMCLK_DELAY_12_ns;
	RamAccess.LoStartAddress = (WORD) (dwStartAddr);
	RamAccess.HiStartAddress = (WORD) (dwStartAddr>>16);
	RamAccess.RwSize = RealTableSize*2;
	RamAccess.BufferPtr = (LPBYTE)lpMotorTable;
	LLFRamAccess(chip,&RamAccess);

	Mustek_SendData(chip,ES01_9D_MotorTableAddrA14_A21, (BYTE)(dwTableBaseAddr>>TABLE_OFFSET_BASE) );		

	dwEndAddr = PackAreaStartAddress - (512*8) - 1;

	WORD wFullBank;
	Mustek_SendData(chip,ES01_FB_BufferEmptySize16WordLSB, LOBYTE(WaitBufferOneLineSize>>(7-3)));
	Mustek_SendData(chip,ES01_FC_BufferEmptySize16WordMSB, HIBYTE(WaitBufferOneLineSize>>(7-3)));

	wFullBank = (WORD)((dwEndAddr - ( ((dwLineWidthPixel*BytePerPixel)/2)*3*1 )) / BANK_SIZE); 
	Mustek_SendData(chip,ES01_F9_BufferFullSize16WordLSB, LOBYTE(wFullBank));
	Mustek_SendData(chip,ES01_FA_BufferFullSize16WordMSB, HIBYTE(wFullBank));

	Mustek_SendData(chip,ES01_DB_PH_RESET_EDGE_TIMING_ADJUST, 0x00);

	LLFSetRamAddress(chip,0x0, dwEndAddr, ACCESS_DRAM); 

	Mustek_SendData(chip,ES01_DC_CLEAR_EDGE_TO_PH_TG_EDGE_WIDTH, 0);

    Mustek_SendData(chip,ES01_00_ADAFEConfiguration,	0x70);
    Mustek_SendData(chip,ES01_02_ADAFEMuxConfig,		0x80);

	
	free(lpMotorTable);
	free(lpMotorStepsTable);

	XDBG(DEBUG_ASIC,"Asic_SetCalibrate: Exit");
	return status;
}


STATUS Asic_SetAFEGainOffset(PAsic chip)
{
	STATUS status = STATUS_GOOD;
	XDBG(DEBUG_ASIC,"Asic_SetAFEGainOffset:Enter");

	status = SetAFEGainOffset(chip);

	XDBG(DEBUG_ASIC,"Asic_SetAFEGainOffset: Exit");
	return status;
}
